# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for get_function_map."""

import typing
from typing import Any
import pytest
from ..._extra_utils import get_function_map
from ...errors import UnsupportedFunctionError
from ...types import GenerateContentConfig

_is_mcp_imported = False
if typing.TYPE_CHECKING:
  from mcp import types as mcp_types
  from mcp import ClientSession as McpClientSession
  from ..._adapters import McpToGenAiToolAdapter

  _is_mcp_imported = True
else:
  McpClientSession: typing.Type = Any
  McpToGenAiToolAdapter: typing.Type = Any
  try:
    from mcp import types as mcp_types
    from mcp import ClientSession as McpClientSession
    from ..._adapters import McpToGenAiToolAdapter

    _is_mcp_imported = True
  except ImportError:
    McpClientSession = None
    McpToGenAiToolAdapter = None


def test_coroutine_function():
  async def func_under_test():
    pass

  config = GenerateContentConfig(tools=[func_under_test])

  with pytest.raises(UnsupportedFunctionError):
    get_function_map(config)


def test_empty_config():
  config = {}

  assert get_function_map(config) == {}


def test_empty_tools():
  config = GenerateContentConfig(top_p=0.5)

  assert get_function_map(config) == {}


def test_valid_function():
  def func_under_test():
    pass

  config = GenerateContentConfig(tools=[func_under_test])

  assert get_function_map(config) == {'func_under_test': func_under_test}


def test_mcp_tool_raises_error():
  if not _is_mcp_imported:
    return

  session = McpClientSession(read_stream=None, write_stream=None)
  config = GenerateContentConfig(tools=[session])
  mcp_to_genai_tool_adapters = {'tool': McpToGenAiToolAdapter(session, [])}
  with pytest.raises(UnsupportedFunctionError):
    get_function_map(
        config, mcp_to_genai_tool_adapters, is_caller_method_async=False
    )


@pytest.mark.asyncio
async def test_mcp_tool():
  if not _is_mcp_imported:
    return

  class MockMcpClientSession(McpClientSession):

    def __init__(self):
      self._read_stream = None
      self._write_stream = None

    async def list_tools(self):
      return mcp_types.ListToolsResult(
          tools=[
              mcp_types.Tool(
                  name='tool',
                  description='tool-description',
                  inputSchema={
                      'type': 'OBJECT',
                      'properties': {
                          'key1': {
                              'type': 'STRING',
                          },
                          'key2': {
                              'type': 'NUMBER',
                          },
                      },
                  },
              )
          ]
      )

  session = MockMcpClientSession()
  config = GenerateContentConfig(tools=[session])
  mcp_to_genai_tool_adapters = {
      'tool': McpToGenAiToolAdapter(session, [await session.list_tools()]),
  }
  result = get_function_map(
      config, mcp_to_genai_tool_adapters, is_caller_method_async=True
  )
  assert isinstance(result['tool'], McpToGenAiToolAdapter)


@pytest.mark.asyncio
async def test_duplicate_mcp_tool_raises_error():
  if not _is_mcp_imported:
    return

  class MockMcpClientSession(McpClientSession):

    def __init__(self):
      self._read_stream = None
      self._write_stream = None

    async def list_tools(self):
      return mcp_types.ListToolsResult(
          tools=[
              mcp_types.Tool(
                  name='tool',
                  description='tool-description',
                  inputSchema={
                      'type': 'OBJECT',
                      'properties': {
                          'key1': {
                              'type': 'STRING',
                          },
                          'key2': {
                              'type': 'NUMBER',
                          },
                      },
                  },
              )
          ]
      )

  def tool():
    pass

  session = MockMcpClientSession()
  config = GenerateContentConfig(tools=[tool, session])
  mcp_to_genai_tool_adapters = {
      'tool': McpToGenAiToolAdapter(session, [await session.list_tools()]),
  }
  with pytest.raises(ValueError):
    get_function_map(
        config, mcp_to_genai_tool_adapters, is_caller_method_async=True
    )



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for batches.create() with inlined requests."""
import base64
import copy
import datetime
import os

import pytest

from ... import _transformers as t
from ... import types
from .. import pytest_helper

_GEMINI_MODEL = 'gemini-2.5-flash'
_DISPLAY_NAME = 'test_batch'

_MLDEV_GEMINI_MODEL = 'gemini-2.5-flash'

_SAFETY_SETTINGS = [
    {
        'category': 'HARM_CATEGORY_HATE_SPEECH',
        'threshold': 'BLOCK_ONLY_HIGH',
    },
    {
        'category': 'HARM_CATEGORY_DANGEROUS_CONTENT',
        'threshold': 'BLOCK_LOW_AND_ABOVE',
    },
]

_INLINED_REQUESTS = [
    {
        'contents': [{
            'parts': [{
                'text': 'what is the number after 1? return just the number.',
            }],
            'role': 'user',
        }],
        'metadata': {
            'key': 'request-1',
        },
        'config': {
            'safety_settings': _SAFETY_SETTINGS,
        },
    },
    {
        'contents': [{
            'parts': [{
                'text': 'what is the number after 2? return just the number.',
            }],
            'role': 'user',
        }],
        'metadata': {
            'key': 'request-2',
        },
        'config': {
            'safety_settings': _SAFETY_SETTINGS,
        },
    },
]
_INLINED_TEXT_REQUEST_UNION = {
    'contents': [{
        'parts': [{
            'text': 'high',
        }],
        'role': 'user',
    }],
    'config': {
        'response_modalities': ['TEXT'],
        'system_instruction': 'I say high, you say low',
        'thinking_config': {
            'include_thoughts': True,
            'thinking_budget': 4000,
        },
    },
}
_INLINED_TEXT_REQUEST = copy.deepcopy(_INLINED_TEXT_REQUEST_UNION)
_INLINED_TEXT_REQUEST['config']['system_instruction'] = t.t_content(
    'I say high, you say low'
)
_INLINED_TEXT_REQUEST['config']['tools'] = [{'google_search': {}}]
_INLINED_TEXT_REQUEST['config']['tool_config'] = {
    'retrieval_config': {'lat_lng': {'latitude': 37.422, 'longitude': -122.084}}
}

_INLINED_IMAGE_REQUEST = {
    'contents': [{
        'parts': [
            {'text': 'What is in this image?'},
            {
                'file_data': {
                    'file_uri': (
                        'https://generativelanguage.googleapis.com/v1beta/files/kje1wewvo85z'
                    ),
                    'mime_type': 'image/jpeg',
                },
            },
        ],
        'role': 'user',
    }],
    'config': {
        'temperature': 0.7,
        'top_p': 0.9,
        'top_k': 10,
    },
}
_INLINED_VIDEO_REQUEST = {
    'contents': [{
        'parts': [
            {
                'text': 'Summerize this video.',
            },
            {
                'file_data': {
                    'file_uri': (
                        'https://generativelanguage.googleapis.com/v1beta/files/tyvaih24jwje'
                    ),
                    'mime_type': 'video/mp4',
                },
                'video_metadata': {
                    'start_offset': '0s',
                    'end_offset': '5s',
                    'fps': 3,
                },
            },
        ],
        'role': 'user',
    }],
}
_IMAGE_PNG_FILE_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../data/google.png')
)
with open(_IMAGE_PNG_FILE_PATH, 'rb') as image_file:
  image_bytes = image_file.read()
  image_string = base64.b64encode(image_bytes).decode('utf-8')
_INLINED_IMAGE_BLOB_REQUEST = types.InlinedRequest(
    contents=[
        types.Content(
            parts=[
                types.Part(text='What is this image about?'),
                types.Part(
                    inline_data=types.Blob(
                        data=image_string,
                        mime_type='image/png',
                    ),
                ),
            ],
            role='user',
        )
    ]
)

# All tests will be run for both Vertex and MLDev.
test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name='test_union_with_inlined_request',
        parameters=types._CreateBatchJobParameters(
            model=_MLDEV_GEMINI_MODEL,
            src=_INLINED_REQUESTS,
            config={
                'display_name': _DISPLAY_NAME,
            },
        ),
        exception_if_vertex='not supported in Vertex',
        has_union=True,
    ),
    pytest_helper.TestTableItem(
        name='test_with_inlined_request',
        parameters=types._CreateBatchJobParameters(
            model=_MLDEV_GEMINI_MODEL,
            src={'inlined_requests': _INLINED_REQUESTS},
            config={
                'display_name': _DISPLAY_NAME,
            },
        ),
        exception_if_vertex='not supported',
    ),
    pytest_helper.TestTableItem(
        name='test_with_inlined_request_config',
        parameters=types._CreateBatchJobParameters(
            model=_MLDEV_GEMINI_MODEL,
            src={'inlined_requests': _INLINED_REQUESTS},
            config={
                'display_name': _DISPLAY_NAME,
            },
        ),
        exception_if_vertex='not supported',
    ),
    pytest_helper.TestTableItem(
        name='test_union_with_inlined_request_system_instruction',
        parameters=types._CreateBatchJobParameters(
            model=_MLDEV_GEMINI_MODEL,
            src={'inlined_requests': [_INLINED_TEXT_REQUEST_UNION]},
            config={
                'display_name': _DISPLAY_NAME,
            },
        ),
        has_union=True,
        exception_if_vertex='not supported',
    ),
    pytest_helper.TestTableItem(
        name='test_with_image_file',
        parameters=types._CreateBatchJobParameters(
            model=_MLDEV_GEMINI_MODEL,
            src={'inlined_requests': [_INLINED_IMAGE_REQUEST]},
            config={
                'display_name': _DISPLAY_NAME,
            },
        ),
        exception_if_vertex='not supported',
    ),
    pytest_helper.TestTableItem(
        name='test_with_image_blob',
        parameters=types._CreateBatchJobParameters(
            model=_MLDEV_GEMINI_MODEL,
            src={'inlined_requests': [_INLINED_IMAGE_BLOB_REQUEST]},
            config={
                'display_name': _DISPLAY_NAME,
            },
        ),
        exception_if_vertex='not supported',
    ),
    pytest_helper.TestTableItem(
        name='test_with_video_file',
        parameters=types._CreateBatchJobParameters(
            model=_MLDEV_GEMINI_MODEL,
            src={'inlined_requests': [_INLINED_VIDEO_REQUEST]},
            config={
                'display_name': _DISPLAY_NAME,
            },
        ),
        exception_if_vertex='not supported',
    ),
]

pytestmark = [
    pytest.mark.usefixtures('mock_timestamped_unique_name'),
    pytest_helper.setup(
        file=__file__,
        globals_for_file=globals(),
        test_method='batches.create',
        test_table=test_table,
    ),
]


@pytest.mark.asyncio
async def test_async_create(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    batch_job = await client.aio.batches.create(
        model=_GEMINI_MODEL,
        src=_INLINED_REQUESTS,
    )
    assert batch_job.name.startswith('batches/')
    assert (
        batch_job.model == 'models/' + _GEMINI_MODEL
    )  # Converted to Gemini full name.



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for http retries."""

import asyncio
from collections.abc import Sequence
import datetime
from unittest import mock
import pytest
try:
    import aiohttp
    AIOHTTP_NOT_INSTALLED = False
except ImportError:
    AIOHTTP_NOT_INSTALLED = True
    aiohttp = mock.MagicMock()

from google.oauth2 import credentials
import httpx
import tenacity

from ... import _api_client as api_client
from ... import errors
from ... import types


requires_aiohttp = pytest.mark.skipif(
    AIOHTTP_NOT_INSTALLED, reason="aiohttp is not installed, skipping test."
)


_RETRIED_CODES = (
    408,  # Request timeout.
    429,  # Too many requests.
    500,  # Internal server error.
    502,  # Bad gateway.
    503,  # Service unavailable.
    504,  # Gateway timeout.
)


@pytest.fixture(autouse=True)
def reset_has_aiohttp():
  yield
  api_client.has_aiohttp = False


def _final_codes(retried_codes: Sequence[int] = _RETRIED_CODES):
  return [code for code in range(100, 600) if code not in retried_codes]


def _httpx_response(code: int):
  return httpx.Response(
      status_code=code,
      headers={'status-code': str(code)},
      content=b'',
  )


# Args


def test_retry_args_disabled():
  args = api_client.retry_args(None)

  assert set(args.keys()) == {'stop', 'reraise'}
  assert args['stop'].max_attempt_number == 1
  assert args['reraise']


def test_retry_args_enabled_with_defaults():
  # Empty options means use the default values whereas None means no retries.
  args = api_client.retry_args(types.HttpRetryOptions())

  assert set(args.keys()) == {
      'stop',
      'retry',
      'wait',
      'reraise',
      'before_sleep',
  }

  assert args['stop'].max_attempt_number == 5

  wait = args['wait']
  assert wait.exp_base == 2
  assert wait.initial == 1
  assert wait.jitter == 1
  assert wait.max == 60

  retry = args['retry']
  for code in _RETRIED_CODES:
    try:
      errors.APIError.raise_for_response(_httpx_response(code))
      assert False, 'Expected APIError to be raised.'
    except errors.APIError as e:
      assert retry.predicate(e)

  for code in _final_codes():
    try:
      errors.APIError.raise_for_response(_httpx_response(code))
      # Does not raise for some codes.
    except errors.APIError as e:
      # Does not retry for error codes outside of the retried codes list.
      assert not retry.predicate(e)

    assert args['reraise']


def test_retry_wait():
  timestamps = []

  def fn():
    now = datetime.datetime.now()
    timestamps.append(now)
    raise errors.APIError.raise_for_response(_httpx_response(429))

  retrying = tenacity.Retrying(
      **api_client.retry_args(types.HttpRetryOptions())
  )

  try:
    retrying(fn)
    assert False, 'Expected APIError to be raised.'
  except errors.APIError:
    pass

  assert len(timestamps) == 5
  assert timestamps[1] - timestamps[0] >= datetime.timedelta(seconds=1)
  assert timestamps[2] - timestamps[1] >= datetime.timedelta(seconds=2)
  assert timestamps[3] - timestamps[2] >= datetime.timedelta(seconds=4)
  assert timestamps[4] - timestamps[3] >= datetime.timedelta(seconds=8)


def test_retry_args_enabled_with_custom_values_are_not_overridden():
  options = types.HttpRetryOptions(
      attempts=10,
      initial_delay=10,
      max_delay=100,
      exp_base=1.5,
      jitter=0.5,
      http_status_codes=[408, 429],
  )
  retry_args = api_client.retry_args(options)
  assert retry_args['stop'].max_attempt_number == 10

  wait = retry_args['wait']
  assert wait.initial == 10
  assert wait.max == 100
  assert wait.exp_base == 1.5
  assert wait.jitter == 0.5

  retry = retry_args['retry']
  for code in [408, 429]:
    try:
      errors.APIError.raise_for_response(_httpx_response(code))
      assert False, 'Expected APIError to be raised.'
    except errors.APIError as e:
      assert retry.predicate(e)

  for code in _final_codes([408, 429]):
    try:
      errors.APIError.raise_for_response(_httpx_response(code))
      # Does not raise for some codes.
    except errors.APIError as e:
      # Does not retry for error codes outside of the retried codes list.
      assert not retry.predicate(e)


def _patch_auth_default():
  return mock.patch(
      'google.auth.default',
      return_value=(credentials.Credentials('magic_token'), 'test_project'),
      autospec=True,
  )


def _transport_options(http_options=None, transport=None, async_transport=None):
  http_options = http_options or types.HttpOptions()
  http_options.client_args = {'transport': transport}
  http_options.async_client_args = {'transport': async_transport}
  return http_options


# Sync


def test_disabled_retries_successful_request_executes_once():
  mock_transport = mock.Mock(spec=httpx.BaseTransport)
  mock_transport.handle_request.return_value = _httpx_response(200)

  client = api_client.BaseApiClient(
      vertexai=True,
      project='test_project',
      location='global',
      http_options=_transport_options(transport=mock_transport),
  )

  with _patch_auth_default():
    response = client.request(http_method='GET', path='path', request_dict={})
    mock_transport.handle_request.assert_called_once()
    assert response.headers['status-code'] == '200'


def test_disabled_retries_failed_request_executes_once():
  mock_transport = mock.Mock(spec=httpx.BaseTransport)
  mock_transport.handle_request.return_value = _httpx_response(429)

  client = api_client.BaseApiClient(
      vertexai=True,
      project='test_project',
      location='global',
      http_options=_transport_options(transport=mock_transport),
  )

  with _patch_auth_default():
    try:
      client.request(http_method='GET', path='path', request_dict={})
      assert False, 'Expected APIError to be raised.'
    except errors.APIError as e:
      assert e.code == 429
    mock_transport.handle_request.assert_called_once()


_RETRY_OPTIONS = types.HttpRetryOptions(
    attempts=2,
    initial_delay=0,
    max_delay=1,
    exp_base=0.1,
    jitter=0.1,
    http_status_codes=[429, 504],
)


def test_retries_successful_request_executes_once():
  mock_transport = mock.Mock(spec=httpx.BaseTransport)
  mock_transport.handle_request.return_value = _httpx_response(200)

  client = api_client.BaseApiClient(
      vertexai=True,
      project='test_project',
      location='global',
      http_options=_transport_options(
          http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
          transport=mock_transport,
      ),
  )

  with _patch_auth_default():
    response = client.request(http_method='GET', path='path', request_dict={})
    mock_transport.handle_request.assert_called_once()
    assert response.headers['status-code'] == '200'


def test_retries_failed_request_retries_successfully():
  mock_transport = mock.Mock(spec=httpx.BaseTransport)
  mock_transport.handle_request.side_effect = (
      _httpx_response(429),
      _httpx_response(200),
  )

  client = api_client.BaseApiClient(
      vertexai=True,
      project='test_project',
      location='global',
      http_options=_transport_options(
          http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
          transport=mock_transport,
      ),
  )

  with _patch_auth_default():
    response = client.request(http_method='GET', path='path', request_dict={})
    mock_transport.handle_request.assert_called()
    assert response.headers['status-code'] == '200'


def test_retries_failed_request_retries_successfully_at_request_level():
  mock_transport = mock.Mock(spec=httpx.BaseTransport)
  mock_transport.handle_request.side_effect = (
      _httpx_response(429),
      _httpx_response(200),
  )

  client = api_client.BaseApiClient(
      vertexai=True,
      project='test_project',
      location='global',
      http_options=_transport_options(
          transport=mock_transport,
      ),
  )

  with _patch_auth_default():
    response = client.request(
        http_method='GET',
        path='path',
        request_dict={},
        http_options=types.HttpOptions(
            retry_options=_RETRY_OPTIONS
        ),  # At request level.
    )
    mock_transport.handle_request.assert_called()
    assert response.headers['status-code'] == '200'


def test_retries_failed_request_retries_unsuccessfully():
  mock_transport = mock.Mock(spec=httpx.BaseTransport)
  mock_transport.handle_request.side_effect = (
      _httpx_response(429),
      _httpx_response(504),
  )

  client = api_client.BaseApiClient(
      vertexai=True,
      project='test_project',
      location='global',
      http_options=_transport_options(
          http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
          transport=mock_transport,
      ),
  )

  with _patch_auth_default():
    try:
      client.request(http_method='GET', path='path', request_dict={})
      assert False, 'Expected APIError to be raised.'
    except errors.APIError as e:
      assert e.code == 504
    mock_transport.handle_request.assert_called()


def test_retries_failed_request_retries_unsuccessfully_at_request_level():
  mock_transport = mock.Mock(spec=httpx.BaseTransport)
  mock_transport.handle_request.side_effect = (
      _httpx_response(429),
      _httpx_response(504),
  )

  client = api_client.BaseApiClient(
      vertexai=True,
      project='test_project',
      location='global',
      http_options=_transport_options(
          transport=mock_transport,
      ),
  )

  with _patch_auth_default():
    try:
      client.request(
          http_method='GET',
          path='path',
          request_dict={},
          http_options={'retry_options': _RETRY_OPTIONS},  # At request level.
      )
      assert False, 'Expected APIError to be raised.'
    except errors.APIError as e:
      assert e.code == 504
    mock_transport.handle_request.assert_called()


# Async httpx


def test_async_disabled_retries_successful_request_executes_once():
  api_client.has_aiohttp = False

  async def run():
    mock_transport = mock.Mock(spec=httpx.AsyncBaseTransport)
    mock_transport.handle_async_request.return_value = _httpx_response(200)

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(async_transport=mock_transport),
    )

    with _patch_auth_default():
      response = await client.async_request(
          http_method='GET', path='path', request_dict={}
      )
      mock_transport.handle_async_request.assert_called_once()
      assert response.headers['status-code'] == '200'

  asyncio.run(run())


def test_async_disabled_retries_failed_request_executes_once():
  api_client.has_aiohttp = False

  async def run():
    mock_transport = mock.Mock(spec=httpx.AsyncBaseTransport)
    mock_transport.handle_async_request.return_value = _httpx_response(429)

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(async_transport=mock_transport),
    )

    with _patch_auth_default():
      try:
        await client.async_request(
            http_method='GET', path='path', request_dict={}
        )
        assert False, 'Expected APIError to be raised.'
      except errors.APIError as e:
        assert e.code == 429
      mock_transport.handle_async_request.assert_called_once()

  asyncio.run(run())


def test_async_retries_successful_request_executes_once():
  api_client.has_aiohttp = False

  async def run():
    mock_transport = mock.Mock(spec=httpx.AsyncBaseTransport)
    mock_transport.handle_async_request.return_value = _httpx_response(200)

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(
            http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
            async_transport=mock_transport,
        ),
    )

    with _patch_auth_default():
      response = await client.async_request(
          http_method='GET', path='path', request_dict={}
      )
      mock_transport.handle_async_request.assert_called_once()
      assert response.headers['status-code'] == '200'

  asyncio.run(run())


def test_async_retries_failed_request_retries_successfully():
  api_client.has_aiohttp = False

  async def run():
    mock_transport = mock.Mock(spec=httpx.AsyncBaseTransport)
    mock_transport.handle_async_request.side_effect = (
        _httpx_response(429),
        _httpx_response(200),
    )

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(
            http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
            async_transport=mock_transport,
        ),
    )

    with _patch_auth_default():
      response = await client.async_request(
          http_method='GET', path='path', request_dict={}
      )
      mock_transport.handle_async_request.assert_called()
      assert response.headers['status-code'] == '200'

  asyncio.run(run())


def test_async_retries_failed_request_retries_successfully_at_request_level():
  api_client.has_aiohttp = False

  async def run():
    mock_transport = mock.Mock(spec=httpx.AsyncBaseTransport)
    mock_transport.handle_async_request.side_effect = (
        _httpx_response(429),
        _httpx_response(200),
    )

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(
            async_transport=mock_transport,
        ),
    )

    with _patch_auth_default():
      response = await client.async_request(
          http_method='GET',
          path='path',
          request_dict={},
          http_options=types.HttpOptions(
              retry_options=_RETRY_OPTIONS
          ),  # At request level.
      )
      mock_transport.handle_async_request.assert_called()
      assert response.headers['status-code'] == '200'

  asyncio.run(run())


def test_async_retries_failed_request_retries_unsuccessfully():
  api_client.has_aiohttp = False

  async def run():
    mock_transport = mock.Mock(spec=httpx.AsyncBaseTransport)
    mock_transport.handle_async_request.side_effect = (
        _httpx_response(429),
        _httpx_response(504),
    )

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(
            http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
            async_transport=mock_transport,
        ),
    )

    with _patch_auth_default():
      try:
        await client.async_request(
            http_method='GET', path='path', request_dict={}
        )
        assert False, 'Expected APIError to be raised.'
      except errors.APIError as e:
        assert e.code == 504
      mock_transport.handle_async_request.assert_called()

  asyncio.run(run())


def test_async_retries_failed_request_retries_unsuccessfully_at_request_level():
  api_client.has_aiohttp = False

  async def run():
    mock_transport = mock.Mock(spec=httpx.AsyncBaseTransport)
    mock_transport.handle_async_request.side_effect = (
        _httpx_response(429),
        _httpx_response(504),
    )

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(
            async_transport=mock_transport,
        ),
    )

    with _patch_auth_default():
      try:
        await client.async_request(
            http_method='GET',
            path='path',
            request_dict={},
            http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
        )
        assert False, 'Expected APIError to be raised.'
      except errors.APIError as e:
        assert e.code == 504
      mock_transport.handle_async_request.assert_called()

  asyncio.run(run())


# Async aiohttp


async def _aiohttp_async_response(status: int):
  """Has to return a coroutine hence async."""
  response = mock.Mock(spec=aiohttp.ClientResponse)
  response.status = status
  response.headers = {'status-code': str(status)}
  response.json.return_value = {}
  response.text.return_value = 'test'
  return response


@requires_aiohttp
@mock.patch.object(aiohttp.ClientSession, 'request', autospec=True)
def test_aiohttp_disabled_retries_successful_request_executes_once(
    mock_request,
):
  api_client.has_aiohttp = True  # Force aiohttp

  async def run():
    mock_request.return_value = _aiohttp_async_response(200)

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
    )

    with _patch_auth_default():
      response = await client.async_request(
          http_method='GET', path='path', request_dict={}
      )
      mock_request.assert_called_once()
      assert response.headers['status-code'] == '200'

  asyncio.run(run())


@requires_aiohttp
@mock.patch.object(aiohttp.ClientSession, 'request', autospec=True)
def test_aiohttp_disabled_retries_failed_request_executes_once(mock_request):
  api_client.has_aiohttp = True

  async def run():
    mock_request.return_value = _aiohttp_async_response(429)

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
    )

    with _patch_auth_default():
      try:
        await client.async_request(
            http_method='GET', path='path', request_dict={}
        )
        assert False, 'Expected APIError to be raised.'
      except errors.APIError as e:
        assert e.code == 429
      mock_request.assert_called_once()

  asyncio.run(run())


@requires_aiohttp
@mock.patch.object(aiohttp.ClientSession, 'request', autospec=True)
def test_aiohttp_retries_successful_request_executes_once(mock_request):
  api_client.has_aiohttp = True

  async def run():
    mock_request.return_value = _aiohttp_async_response(200)

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(
            http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
        ),
    )

    with _patch_auth_default():
      response = await client.async_request(
          http_method='GET', path='path', request_dict={}
      )
      mock_request.assert_called_once()
      assert response.headers['status-code'] == '200'

  asyncio.run(run())


@requires_aiohttp
@mock.patch.object(aiohttp.ClientSession, 'request', autospec=True)
def test_aiohttp_retries_failed_request_retries_successfully(mock_request):
  api_client.has_aiohttp = True

  async def run():
    mock_request.side_effect = (
        _aiohttp_async_response(429),
        _aiohttp_async_response(200),
    )

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(
            http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
        ),
    )

    with _patch_auth_default():
      response = await client.async_request(
          http_method='GET', path='path', request_dict={}
      )
      mock_request.assert_called()
      assert response.headers['status-code'] == '200'

  asyncio.run(run())


@requires_aiohttp
@mock.patch.object(aiohttp.ClientSession, 'request', autospec=True)
def test_aiohttp_retries_failed_request_retries_successfully_at_request_level(
    mock_request,
):
  api_client.has_aiohttp = True

  async def run():
    mock_request.side_effect = (
        _aiohttp_async_response(429),
        _aiohttp_async_response(200),
    )

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
    )

    with _patch_auth_default():
      response = await client.async_request(
          http_method='GET',
          path='path',
          request_dict={},
          http_options=types.HttpOptions(
              retry_options=_RETRY_OPTIONS
          ),  # At request level.
      )
      mock_request.assert_called()
      assert response.headers['status-code'] == '200'

  asyncio.run(run())


@requires_aiohttp
@mock.patch.object(aiohttp.ClientSession, 'request', autospec=True)
def test_aiohttp_retries_failed_request_retries_unsuccessfully(mock_request):
  api_client.has_aiohttp = True

  async def run():
    mock_request.side_effect = (
        _aiohttp_async_response(429),
        _aiohttp_async_response(504),
    )

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
        http_options=_transport_options(
            http_options=types.HttpOptions(retry_options=_RETRY_OPTIONS),
        ),
    )

    with _patch_auth_default():
      try:
        await client.async_request(
            http_method='GET', path='path', request_dict={}
        )
        assert False, 'Expected APIError to be raised.'
      except errors.APIError as e:
        assert e.code == 504
      mock_request.assert_called()

  asyncio.run(run())


@requires_aiohttp
@mock.patch.object(aiohttp.ClientSession, 'request', autospec=True)
def test_aiohttp_retries_failed_request_retries_unsuccessfully_at_request_level(
    mock_request,
):
  api_client.has_aiohttp = True

  async def run():
    mock_request.side_effect = (
        _aiohttp_async_response(429),
        _aiohttp_async_response(504),
    )

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
    )

    with _patch_auth_default():
      try:
        await client.async_request(
            http_method='GET',
            path='path',
            request_dict={},
            http_options={'retry_options': _RETRY_OPTIONS},  # At request level.
        )
        assert False, 'Expected APIError to be raised.'
      except errors.APIError as e:
        assert e.code == 504
      mock_request.assert_called()

  asyncio.run(run())


@requires_aiohttp
@mock.patch.object(aiohttp.ClientSession, 'request', autospec=True)
def test_aiohttp_retries_client_connector_error_retries_successfully(
    mock_request,
):
  api_client.has_aiohttp = True

  async def run():
    mock_request.side_effect = (
        aiohttp.ClientConnectorError(
            connection_key=aiohttp.client_reqrep.ConnectionKey(
                'localhost', 80, False, True, None, None, None
            ),
            os_error=OSError,
        ),
        _aiohttp_async_response(200),
    )
    # The request will be automatically retried once, if catching the
    # ClientConnectorError.

    client = api_client.BaseApiClient(
        vertexai=True,
        project='test_project',
        location='global',
    )

    with _patch_auth_default():
      response = await client.async_request(
          http_method='GET', path='path', request_dict={}
      )
      mock_request.assert_called()
      assert response.headers['status-code'] == '200'

  asyncio.run(run())



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Test convert_if_exist_pydantic_model."""

import inspect
from typing import Optional, Union
import pydantic
import pytest
import sys
from ... import errors
from ..._extra_utils import convert_if_exist_pydantic_model


def test_builtin_types():
  assert convert_if_exist_pydantic_model(1, int, 'param_name', 'func_name') == 1
  assert (
      convert_if_exist_pydantic_model(1.0, float, 'param_name', 'func_name')
      == 1.0
  )
  assert (
      convert_if_exist_pydantic_model('1.0', str, 'param_name', 'func_name')
      == '1.0'
  )
  assert (
      convert_if_exist_pydantic_model(True, bool, 'param_name', 'func_name')
      is True
  )
  assert convert_if_exist_pydantic_model(
      [1], list, 'param_name', 'func_name'
  ) == [1]
  assert convert_if_exist_pydantic_model(
      {'key1': 1}, dict, 'param_name', 'func_name'
  ) == {'key1': 1}
  assert convert_if_exist_pydantic_model(
      {'key1': 1, 'key2': 2}, dict[str, int], 'param_name', 'func_name'
  ) == {'key1': 1, 'key2': 2}


def test_value_int_annotation_float():
  assert (
      convert_if_exist_pydantic_model(1, float, 'param_name', 'func_name')
      == 1.0
  )


def test_union_types():
  assert (
      convert_if_exist_pydantic_model(
          1, Union[int, float], 'param_name', 'func_name'
      )
      == 1
  )
  assert (
      convert_if_exist_pydantic_model(
          1.0, Union[int, float], 'param_name', 'func_name'
      )
      == 1.0
  )
  assert (
      convert_if_exist_pydantic_model(
          '1.0', Union[str, float], 'param_name', 'func_name'
      )
      == '1.0'
  )
  assert (
      convert_if_exist_pydantic_model(
          True, Union[bool, str], 'param_name', 'func_name'
      )
      is True
  )

  # | in 3.10+
  if sys.version_info >= (3, 10):
    assert (
        convert_if_exist_pydantic_model(1, int | float, 'param_name', 'func_name')
        == 1
    )


def test_nested_pydantic_model():
  class SimpleModel(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  class ComplexModel(pydantic.BaseModel):
    key1_complex: SimpleModel
    key2_complex: list[SimpleModel]
    key3_complex: dict[str, SimpleModel]

  def foo(_: ComplexModel):
    pass

  annotation = inspect.signature(foo).parameters['_'].annotation
  original_args = {
      'key1_complex': {'key1_simple': 1, 'key2_simple': 1.0},
      'key2_complex': [
          {'key1_simple': 2, 'key2_simple': 2.0},
          {'key1_simple': 3, 'key2_simple': 3.0},
      ],
      'key3_complex': {
          'key1_simple': {'key1_simple': 4, 'key2_simple': 4.0},
          'key2_simple': {'key1_simple': 5, 'key2_simple': 5.0},
      },
  }

  converted_args = convert_if_exist_pydantic_model(
      original_args, annotation, 'param_name', 'func_name'
  )
  assert isinstance(converted_args, ComplexModel)
  assert isinstance(converted_args.key1_complex, SimpleModel)
  assert isinstance(converted_args.key2_complex[0], SimpleModel)
  assert isinstance(converted_args.key2_complex[1], SimpleModel)
  assert isinstance(converted_args.key3_complex['key1_simple'], SimpleModel)
  assert isinstance(converted_args.key3_complex['key2_simple'], SimpleModel)
  assert converted_args.model_dump() == original_args


def test_pydantic_model_in_list_union_type():
  class SimpleModel(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  def foo(_: list[Union[int, SimpleModel]]):
    pass

  annotation = inspect.signature(foo).parameters['_'].annotation
  original_args = [1, {'key1_simple': 1, 'key2_simple': 1.0}]

  converted_args = convert_if_exist_pydantic_model(
      original_args, annotation, 'param_name', 'func_name'
  )

  assert isinstance(converted_args, list)
  assert len(converted_args) == 2
  assert isinstance(converted_args[0], int)
  assert isinstance(converted_args[1], SimpleModel)
  assert converted_args[0] == original_args[0]
  assert converted_args[1].model_dump() == original_args[1]


def test_generic_list_type():
  def foo(_: Optional[list[Union[int, float]]]):
    pass

  annotation = inspect.signature(foo).parameters['_'].annotation
  original_args = [1, 1.5]

  converted_args = convert_if_exist_pydantic_model(
      original_args, annotation, 'param_name', 'func_name'
  )

  assert isinstance(converted_args, list)
  assert len(converted_args) == 2
  assert isinstance(converted_args[0], int)
  assert isinstance(converted_args[1], float)
  assert converted_args[0] == original_args[0]


@pytest.mark.skipif(sys.version_info < (3, 10), reason='need python 3.10+')
def test_pydantic_model_with_union_and_generic_type():
  class Model1(pydantic.BaseModel):
    arg1: Union[int, float]
    arg2: list[str]
    arg3: int | float  # python 3.9+
    arg4: Union[list[str], str]

  def foo(_: Optional[list[Union[int, Model1]]]):
    pass

  annotation = inspect.signature(foo).parameters['_'].annotation
  original_args = [1, {'arg1': 1, 'arg2': ['a'], 'arg3': 1.0, 'arg4': '1.0'}]

  converted_args = convert_if_exist_pydantic_model(
      original_args, annotation, 'param_name', 'func_name'
  )

  assert isinstance(converted_args, list)
  assert len(converted_args) == 2
  assert isinstance(converted_args[0], int)
  assert isinstance(converted_args[1], Model1)
  assert converted_args[0] == original_args[0]


def test_unknown_pydantic_model_argument_error():
  class SimpleModel(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  def foo(_: SimpleModel):
    pass

  annotation = inspect.signature(foo).parameters['_'].annotation
  original_args = {'key3_simple': 1, 'key2_simple': 1.0}

  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        original_args, annotation, 'param_name', 'func_name'
    )


def test_unknown_pydantic_model_argument_error_with_union_type():
  class SimpleModel1(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  class SimpleModel2(pydantic.BaseModel):
    key3_simple: str
    key4_simple: float

  def foo(_: Union[SimpleModel1, SimpleModel2]):
    pass

  annotation = inspect.signature(foo).parameters['_'].annotation
  original_args = {'key5_simple': 1, 'key4_simple': 1.0}

  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        original_args, annotation, 'param_name', 'func_name'
    )


def test_unknown_pydantic_model_argument_error_with_union_type_and_builtin_type():
  class SimpleModel1(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  def foo(_: Union[SimpleModel1, int]):
    pass

  annotation = inspect.signature(foo).parameters['_'].annotation
  original_args = {'key5_simple': 1, 'key4_simple': 1.0}

  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        original_args, annotation, 'param_name', 'func_name'
    )


def test_incompatible_value_and_annotation():
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        {'key1_simple': 1, 'key2_simple': 1.0},
        int,
        'param_name',
        'func_name',
    )
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        {'key1_simple': 1, 'key2_simple': 1.0},
        float,
        'param_name',
        'func_name',
    )
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        1,
        str,
        'param_name',
        'func_name',
    )
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        1.0,
        str,
        'param_name',
        'func_name',
    )
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        True,
        str,
        'param_name',
        'func_name',
    )
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        [1],
        str,
        'param_name',
        'func_name',
    )
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        {'key1': 1},
        str,
        'param_name',
        'func_name',
    )
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    convert_if_exist_pydantic_model(
        {'key1': 1, 'key2': 2},
        str,
        'param_name',
        'func_name',
    )



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .function_param import FunctionParam
from .allowed_tools_param import AllowedToolsParam

__all__ = ["ToolParam", "GoogleSearch", "CodeExecution", "URLContext", "ComputerUse", "MCPServer", "FileSearch"]


class GoogleSearch(TypedDict, total=False):
    """A tool that can be used by the model to search Google."""

    type: Required[Literal["google_search"]]

    search_types: List[Literal["web_search", "image_search"]]
    """The types of search grounding to enable."""


class CodeExecution(TypedDict, total=False):
    """A tool that can be used by the model to execute code."""

    type: Required[Literal["code_execution"]]


class URLContext(TypedDict, total=False):
    """A tool that can be used by the model to fetch URL context."""

    type: Required[Literal["url_context"]]


class ComputerUse(TypedDict, total=False):
    """A tool that can be used by the model to interact with the computer."""

    type: Required[Literal["computer_use"]]

    environment: Literal["browser"]
    """The environment being operated."""

    excluded_predefined_functions: Annotated[SequenceNotStr[str], PropertyInfo(alias="excludedPredefinedFunctions")]
    """The list of predefined functions that are excluded from the model call."""


class MCPServer(TypedDict, total=False):
    """A MCPServer is a server that can be called by the model to perform actions."""

    type: Required[Literal["mcp_server"]]

    allowed_tools: Iterable[AllowedToolsParam]
    """The allowed tools."""

    headers: Dict[str, str]
    """Optional: Fields for authentication headers, timeouts, etc., if needed."""

    name: str
    """The name of the MCPServer."""

    url: str
    """
    The full URL for the MCPServer endpoint.
    Example: "https://api.example.com/mcp"
    """


class FileSearch(TypedDict, total=False):
    """A tool that can be used by the model to search files."""

    type: Required[Literal["file_search"]]

    file_search_store_names: SequenceNotStr[str]
    """The file search store names to search."""

    metadata_filter: str
    """Metadata filter to apply to the semantic retrieval documents and chunks."""

    top_k: int
    """The number of semantic retrieval chunks to retrieve."""


ToolParam: TypeAlias = Union[FunctionParam, GoogleSearch, CodeExecution, URLContext, ComputerUse, MCPServer, FileSearch]



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for upscale_image."""

import os

from pydantic import ValidationError
import pytest

from ... import types
from .. import pytest_helper

IMAGEN_MODEL_LATEST = 'imagen-4.0-upscale-preview'

IMAGE_FILE_PATH = os.path.abspath(
    os.path.join(os.path.dirname(__file__), '../data/bridge1.png')
)

test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name='test_upscale_no_config',
        exception_if_mldev='only supported in the Vertex AI client',
        parameters=types.UpscaleImageParameters(
            model=IMAGEN_MODEL_LATEST,
            image=types.Image.from_file(location=IMAGE_FILE_PATH),
            upscale_factor='x2',
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_upscale',
        exception_if_mldev='only supported in the Vertex AI client',
        parameters=types.UpscaleImageParameters(
            model=IMAGEN_MODEL_LATEST,
            image=types.Image.from_file(location=IMAGE_FILE_PATH),
            upscale_factor='x2',
            config=types.UpscaleImageConfig(
                include_rai_reason=True,
                person_generation=types.PersonGeneration.ALLOW_ADULT,
                safety_filter_level=types.SafetyFilterLevel.BLOCK_LOW_AND_ABOVE,
                output_mime_type='image/jpeg',
                output_compression_quality=80,
                enhance_input_image=True,
                image_preservation_factor=0.6,
                labels={'imagen_label_key': 'upscale_image'}
            ),
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_upscale_gcs',
        exception_if_mldev='only supported in the Vertex AI client',
        parameters=types.UpscaleImageParameters(
            model=IMAGEN_MODEL_LATEST,
            image=types.Image.from_file(location=IMAGE_FILE_PATH),
            upscale_factor='x2',
            config=types.UpscaleImageConfig(
                output_gcs_uri='gs://genai-sdk-tests/temp/images/',
            ),
        ),
    ),
]
pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='models.upscale_image',
    test_table=test_table,
)


def test_upscale_extra_config_parameters(client):
  # MLDev currently does not support upscale_image, but the ValidationError
  # occurs before the ValueError.
  try:
    # User is not allowed to set mode or number_of_images
    client.models.upscale_image(
        model=IMAGEN_MODEL_LATEST,
        image=types.Image.from_file(location=IMAGE_FILE_PATH),
        upscale_factor='x2',
        config={
            'mode': 'upscale',
            'number_of_images': 1,
        }
    )
    # Should never reach this.
    assert False
  except Exception as e:
    assert isinstance(e, ValidationError)
    assert 'Extra inputs are not permitted' in str(e)


@pytest.mark.asyncio
async def test_upscale_async(client):
  with pytest_helper.exception_if_mldev(client, ValueError):
    response = await client.aio.models.upscale_image(
        model=IMAGEN_MODEL_LATEST,
        image=types.Image.from_file(location=IMAGE_FILE_PATH),
        upscale_factor='x2',
        config=types.UpscaleImageConfig(
            person_generation=types.PersonGeneration.ALLOW_ADULT,
            safety_filter_level=types.SafetyFilterLevel.BLOCK_LOW_AND_ABOVE,
            include_rai_reason=True,
            output_mime_type='image/jpeg',
            output_compression_quality=80,
            enhance_input_image=True,
            image_preservation_factor=0.6,
        ),
    )
    assert response.generated_images[0].image.image_bytes


@pytest.mark.asyncio
async def test_upscale_gcs_async(client):
  with pytest_helper.exception_if_mldev(client, ValueError):
    response = await client.aio.models.upscale_image(
        model=IMAGEN_MODEL_LATEST,
        image=types.Image.from_file(location=IMAGE_FILE_PATH),
        upscale_factor='x2',
        config=types.UpscaleImageConfig(
            output_gcs_uri='gs://genai-sdk-tests/temp/images/',
        ),
    )
    assert response.generated_images[0].image.gcs_uri


@pytest.mark.asyncio
async def test_upscale_extra_config_parameters_async(client):
  # MLDev currently does not support upscale_image, but the ValidationError
  # occurs before the ValueError.
  try:
    # User is not allowed to set mode or number_of_images
    await client.aio.models.upscale_image(
        model=IMAGEN_MODEL_LATEST,
        image=types.Image.from_file(location=IMAGE_FILE_PATH),
        upscale_factor='x2',
        config={
            'mode': 'upscale',
            'number_of_images': 1,
        },
    )
    # Should never reach this.
    assert False
  except Exception as e:
    assert isinstance(e, ValidationError)
    assert 'Extra inputs are not permitted' in str(e)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for client behavior when issuing requests."""

import pytest

from ... import _api_client
from ... import types


def test_patch_http_options_with_copies_all_fields():
  patch_options = types.HttpOptions(
      base_url='https://fake-url.com/',
      api_version='v1',
      headers={'X-Custom-Header': 'custom_value'},
      timeout=10000,
      client_args={'http2': True},
      async_client_args={'http1': True},
      extra_body={'key': 'value'},
      retry_options=types.HttpRetryOptions(attempts=10),
      base_url_resource_scope=types.ResourceScope.COLLECTION,
  )
  options = types.HttpOptions()
  patched = _api_client.patch_http_options(options, patch_options)
  http_options_keys = types.HttpOptions.model_fields.keys()

  for key in http_options_keys:
    assert hasattr(patched, key)
    if key not in [
        'httpx_client',
        'httpx_async_client',
        'aiohttp_client',
    ]:
      assert getattr(patched, key) is not None
  assert patched.base_url == 'https://fake-url.com/'
  assert patched.api_version == 'v1'
  assert patched.headers['X-Custom-Header'] == 'custom_value'
  assert patched.timeout == 10000
  assert patched.retry_options.attempts == 10
  assert patched.client_args['http2']
  assert patched.async_client_args['http1']


def test_patch_http_options_merges_headers():
  original_options = types.HttpOptions(
      headers={
          'X-Custom-Header': 'different_value',
          'X-different-header': 'different_value',
      }
  )
  patch_options = types.HttpOptions(
      base_url='https://fake-url.com/',
      api_version='v1',
      headers={'X-Custom-Header': 'custom_value'},
      timeout=10000,
  )
  patched = _api_client.patch_http_options(original_options, patch_options)
  # If the header is present in both the original and patch options, the patch
  # options value should be used
  assert patched.headers['X-Custom-Header'] == 'custom_value'
  assert patched.headers['X-different-header'] == 'different_value'
  assert patched.base_url == 'https://fake-url.com/'
  assert patched.api_version == 'v1'
  assert patched.timeout == 10000


def test_patch_http_options_appends_version_headers():
  original_options = types.HttpOptions(
      headers={
          'X-Custom-Header': 'different_value',
          'X-different-header': 'different_value',
      }
  )
  patch_options = types.HttpOptions(
      base_url='https://fake-url.com/',
      api_version='v1',
      headers={'X-Custom-Header': 'custom_value'},
      timeout=10000,
  )
  patched = _api_client.patch_http_options(original_options, patch_options)
  assert 'user-agent' in patched.headers
  assert 'x-goog-api-client' in patched.headers


def test_setting_timeout_populates_server_timeout_header():
  api_client = _api_client.BaseApiClient(
      vertexai=False,
      api_key='test_api_key',
      http_options=types.HttpOptions(timeout=10000),
  )
  request = api_client._build_request(
      http_method='POST',
      path='sample/path',
      request_dict={},
  )
  assert 'X-Server-Timeout' in request.headers
  assert request.headers['X-Server-Timeout'] == '10'


def test_timeout_rounded_to_nearest_second():
  api_client = _api_client.BaseApiClient(
      vertexai=False,
      api_key='test_api_key',
  )
  http_options = types.HttpOptions(timeout=7300)
  request = api_client._build_request(
      http_method='POST',
      path='sample/path',
      request_dict={},
      http_options=http_options,
  )
  assert request.headers['X-Server-Timeout'] == '8'


def test_server_timeout_not_overwritten():
  api_client = _api_client.BaseApiClient(
      vertexai=False,
      api_key='test_api_key',
  )
  http_options = types.HttpOptions(
      headers={'X-Server-Timeout': '3'},
      timeout=11000)
  request = api_client._build_request(
      http_method='POST',
      path='sample/path',
      request_dict={},
      http_options=http_options,
  )
  assert request.headers['X-Server-Timeout'] == '3'


def test_server_timeout_not_set_by_default():
  api_client = _api_client.BaseApiClient(
      vertexai=False,
      api_key='test_api_key',
  )
  request = api_client._build_request(
      http_method='POST',
      path='sample/path',
      request_dict={},
  )
  assert not 'X-Server-Timeout' in request.headers


def test_resource_scope_without_base_url_raises_error():
  with pytest.raises(ValueError):
    _api_client.BaseApiClient(
        vertexai=True,
        http_options=types.HttpOptions(
            base_url_resource_scope=types.ResourceScope.COLLECTION,
        ),
    )


def test_base_url_resource_scope_not_set_by_default():
  api_client = _api_client.BaseApiClient(
      vertexai=True,
      http_options=types.HttpOptions(
          base_url='https://fake-url.com/',
      ),
  )

  assert api_client._http_options.base_url_resource_scope is None


def test_retry_options_not_set_by_default():
  options = types.HttpOptions()
  assert options.retry_options is None



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


import pytest
from ... import types
from .. import pytest_helper
from . import constants


test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        skip_in_api_mode='Delete is not reproducible in the API mode.',
        name='test_caches_delete_with_vertex_cache_name',
        exception_if_mldev='404',
        parameters=types._DeleteCachedContentParameters(
            name=constants.CACHED_CONTENT_NAME_VERTEX,
            config={
                'http_options': constants.VERTEX_HTTP_OPTIONS,
            },
        ),
    ),
    pytest_helper.TestTableItem(
        skip_in_api_mode='Delete is not reproducible in the API mode.',
        name='test_caches_delete_with_mldev_cache_name',
        exception_if_vertex='404',
        parameters=types._DeleteCachedContentParameters(
            name=constants.CACHED_CONTENT_NAME_MLDEV,
            config={
                'http_options': constants.MLDEV_HTTP_OPTIONS,
            },
        ),
    ),
]
pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='caches.delete',
    test_table=test_table,
)



import pytest
from .. import pytest_helper

pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='models.generate_content',
)


def test_simple_request(client):
  # TODO(b/388917450): Add Vertex AI in Express mode test suite
  client._api_client.project = None
  client._api_client.location = None

  # To record a replay file, replace with api key (from Vertex AI Express).
  # API mode will not work if the API key is a ML Dev API key.
  # After recording, change the string back to 'key'.
  client._api_client._http_options.headers['x-goog-api-key'] = 'key'
  if not client._api_client.vertexai:
    return
  response = client.models.generate_content(
      model='gemini-2.0-flash-001', contents='Tell me a joke.'
  )
  assert response.text


def test_simple_request_stream(client):
  # TODO(b/388917450): Add Vertex AI in Express mode test suite
  client._api_client.project = None
  client._api_client.location = None

  # To record a replay file, replace with api key (from Vertex AI Express).
  # API mode will not work if the API key is a ML Dev API key.
  # After recording, change the string back to 'key'.
  client._api_client._http_options.headers['x-goog-api-key'] = 'key'
  if not client._api_client.vertexai:
    return

  response = client.models.generate_content_stream(
      model='gemini-2.0-flash-001', contents='Tell me a joke.'
  )

  assert any(chunk.text for chunk in response)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for file_search_stores.upload_to_file_search_store()."""

import io
import pathlib
import pytest
from ... import types
from .. import pytest_helper

_FILE_SEARCH_STORE_NAME = 'fileSearchStores/my-store-37cbhu1nw16r'
_FILE_NAME = 'files/mk4h34zkv33d'

test_table: list[pytest_helper.TestTableItem] = []
pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='filesearchstores.import_file',
    test_table=test_table,
)


def test_import(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    operation = client.file_search_stores.import_file(
        file_search_store_name=_FILE_SEARCH_STORE_NAME, file_name=_FILE_NAME
    )


def test_chunking(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    operation = client.file_search_stores.import_file(
        file_search_store_name=_FILE_SEARCH_STORE_NAME,
        file_name=_FILE_NAME,
        config=types.ImportFileConfig(
            chunking_config=types.ChunkingConfig(
                white_space_config=types.WhiteSpaceConfig(
                    max_tokens_per_chunk=200,
                    max_overlap_tokens=20,
                )
            ),
        ),
    )


def test_metadata(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    operation = client.file_search_stores.import_file(
        file_search_store_name=_FILE_SEARCH_STORE_NAME,
        file_name=_FILE_NAME,
        config=types.ImportFileConfig(
            custom_metadata=[
                types.CustomMetadata(key='year', numeric_value=2024),
                types.CustomMetadata(key='tag', string_value='story'),
            ],
        ),
    )


@pytest.mark.asyncio
async def test_async_import(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    operation = await client.aio.file_search_stores.import_file(
        file_search_store_name=_FILE_SEARCH_STORE_NAME,
        file_name=_FILE_NAME,
    )


@pytest.mark.asyncio
async def test_async_chunking(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    operation = await client.aio.file_search_stores.import_file(
        file_search_store_name=_FILE_SEARCH_STORE_NAME,
        file_name=_FILE_NAME,
        config=types.ImportFileConfig(
            chunking_config=types.ChunkingConfig(
                white_space_config=types.WhiteSpaceConfig(
                    max_tokens_per_chunk=200,
                    max_overlap_tokens=20,
                )
            ),
        ),
    )


@pytest.mark.asyncio
async def test_async_metadata(client):
  with pytest_helper.exception_if_vertex(client, ValueError):
    operation = await client.aio.file_search_stores.import_file(
        file_search_store_name=_FILE_SEARCH_STORE_NAME,
        file_name=_FILE_NAME,
        config=types.ImportFileConfig(
            custom_metadata=[
                types.CustomMetadata(key='year', numeric_value=2024),
                types.CustomMetadata(key='tag', string_value='story'),
            ],
        ),
    )



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


from ... import _transformers as t
from ... import types
from .. import pytest_helper

VERTEX_CACHED_CONTENT_NAME = '8163416782997028864'
MLDEV_CACHED_CONTENT_NAME = 'idv6zldt3p2f1lbm31vn41tx2ld42qngpcnka2s3'

test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name='test_cached_content_wrong_name',
        exception_if_vertex='INVALID_ARGUMENT',
        exception_if_mldev='INVALID_ARGUMENT',
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=t.t_contents('What is in these docs?'),
            config={
                'cached_content': 'batchPredictionJobs/123',
            },
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_cached_content_partial_vertex_resource_name_0',
        exception_if_mldev='INVALID_ARGUMENT',
        skip_in_api_mode=(
            'CachedContent API has expiration and permission issues'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=t.t_contents('What is in these docs?'),
            config={
                'cached_content': f'locations/us-central1/cachedContents/{VERTEX_CACHED_CONTENT_NAME}',
            },
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_cached_content_partial_vertex_resource_name_1',
        exception_if_mldev='PERMISSION_DENIED',
        skip_in_api_mode=(
            'CachedContent API has expiration and permission issues'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=t.t_contents('What is in these docs?'),
            config={
                'cached_content': (
                    f'cachedContents/{VERTEX_CACHED_CONTENT_NAME}'
                ),
            },
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_cached_content_partial_vertex_resource_name_2',
        exception_if_mldev='PERMISSION_DENIED',
        skip_in_api_mode=(
            'CachedContent API has expiration and permission issues'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=t.t_contents('What is in these docs?'),
            config={
                'cached_content': VERTEX_CACHED_CONTENT_NAME,
            },
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_cached_content_partial_mldev_resource_name_1',
        exception_if_vertex='NOT_FOUND',
        skip_in_api_mode=(
            'CachedContent API has expiration and permission issues'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=t.t_contents('What is in these docs?'),
            config={
                'cached_content': f'{MLDEV_CACHED_CONTENT_NAME}',
            },
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_cached_content_for_vertex',
        exception_if_mldev='INVALID_ARGUMENT',
        skip_in_api_mode=(
            'CachedContent API has expiration and permission issues'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=t.t_contents('What is in these docs?'),
            config={
                'cached_content': f'projects/964831358985/locations/us-central1/cachedContents/{VERTEX_CACHED_CONTENT_NAME}',
            },
        ),
    ),
    pytest_helper.TestTableItem(
        name='test_cached_content_for_mldev',
        exception_if_vertex='NOT_FOUND',
        skip_in_api_mode=(
            'CachedContent API has expiration and permission issues'
        ),
        parameters=types._GenerateContentParameters(
            model='gemini-2.5-flash',
            contents=t.t_contents('Tell me a story in 300 words.'),
            config={
                'cached_content': f'cachedContents/{MLDEV_CACHED_CONTENT_NAME}',
            },
        ),
    ),
]


pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='models.generate_content',
    test_table=test_table,
)
pytest_plugins = ('pytest_asyncio',)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for t_content."""

import pytest
import pydantic

from ... import _transformers as t
from ... import types


def test_none():
  with pytest.raises(ValueError):
    t.t_content( None)


def test_content():
  assert t.t_content(
      types.Content(parts=[types.Part(text='test')])
  ) == types.Content(parts=[types.Part(text='test')])


def test_content_dict():
  assert t.t_content(
      {'role': 'user', 'parts': [{'text': 'test'}]}
  ) == types.Content(parts=[types.Part(text='test')], role='user')


def test_content_dict_invalid():
  with pytest.raises(pydantic.ValidationError):
    t.t_content({'invalid_key': 'test'})


def test_text_part_dict():
  assert t.t_content({'text': 'test'}) == types.UserContent(
      parts=[types.Part(text='test')]
  )


def test_function_call_part_dict():
  assert t.t_content(
      {'function_call': {'name': 'test_func', 'args': {'arg1': 'value1'}}}
  ) == (
      types.ModelContent(
          parts=[
              types.Part(
                  function_call=types.FunctionCall(
                      name='test_func', args={'arg1': 'value1'}
                  )
              )
          ]
      )
  )


def test_text_part():
  assert t.t_content(types.Part(text='test')) == types.UserContent(
      parts=[types.Part(text='test')]
  )


def test_function_call_part():
  assert t.t_content(
      types.Part(
          function_call=types.FunctionCall(
              name='test_func', args={'arg1': 'value1'}
          )
      ),
  ) == (
      types.ModelContent(
          parts=[
              types.Part(
                  function_call=types.FunctionCall(
                      name='test_func', args={'arg1': 'value1'}
                  )
              )
          ]
      )
  )


def test_string():
  assert t.t_content('test') == types.UserContent(
      parts=[types.Part(text='test')]
  )


def test_file():
  assert t.t_content(
      types.File(uri='gs://test', mime_type='image/png')
  ) == types.UserContent(
      parts=[
          types.Part(
              file_data=types.FileData(
                  file_uri='gs://test', mime_type='image/png'
              )
          )
      ]
  )


def test_file_no_uri():
  with pytest.raises(ValueError):
    t.t_content(types.File(mime_type='image/png'))


def test_file_no_mime_type():
  with pytest.raises(ValueError):
    t.t_content(types.File(uri='gs://test'))


def test_file_dict():
  assert t.t_content({'file_uri': 'gs://test', 'mime_type': 'image/png'}) == types.UserContent(
          parts=[
              types.Part(
                  file_data=types.FileData(
                      file_uri='gs://test', mime_type='image/png'
                  )
              )
          ]
      )

def test_int():
  try:
    t.t_content(1)
  except ValueError as e:
    assert 'Unsupported content part type: <class \'int\'>' in str(e)


@pytest.mark.parametrize(
    'contents',
    [
        types.Content(parts=[types.Part(inline_data=types.Blob(data=b'test'))]),
        {'parts': [{'inline_data': {'data': b'test'}}]},
        [
            types.Content(
                parts=[types.Part(inline_data=types.Blob(data=b'test'))]
            )
        ],
        [{'parts': [{'inline_data': {'data': b'test'}}]}],
    ],
)
def test_t_contents_strict_content(contents):
  result = t.t_contents_strict(contents)
  assert result == [
      types.Content(parts=[types.Part(inline_data=types.Blob(data=b'test'))])
  ]



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Code generated by the Google Gen AI SDK generator DO NOT EDIT.

from typing import Any, Optional, Union

from . import _transformers as t
from ._api_client import BaseApiClient
from ._common import get_value_by_path as getv
from ._common import set_value_by_path as setv


def _Blob_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['data']) is not None:
    setv(to_object, ['data'], getv(from_object, ['data']))

  if getv(from_object, ['display_name']) is not None:
    raise ValueError('display_name parameter is not supported in Gemini API.')

  if getv(from_object, ['mime_type']) is not None:
    setv(to_object, ['mimeType'], getv(from_object, ['mime_type']))

  return to_object


def _Content_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['parts']) is not None:
    setv(
        to_object,
        ['parts'],
        [
            _Part_to_mldev(item, to_object)
            for item in getv(from_object, ['parts'])
        ],
    )

  if getv(from_object, ['role']) is not None:
    setv(to_object, ['role'], getv(from_object, ['role']))

  return to_object


def _FileData_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['display_name']) is not None:
    raise ValueError('display_name parameter is not supported in Gemini API.')

  if getv(from_object, ['file_uri']) is not None:
    setv(to_object, ['fileUri'], getv(from_object, ['file_uri']))

  if getv(from_object, ['mime_type']) is not None:
    setv(to_object, ['mimeType'], getv(from_object, ['mime_type']))

  return to_object


def _FunctionCall_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['id']) is not None:
    setv(to_object, ['id'], getv(from_object, ['id']))

  if getv(from_object, ['args']) is not None:
    setv(to_object, ['args'], getv(from_object, ['args']))

  if getv(from_object, ['name']) is not None:
    setv(to_object, ['name'], getv(from_object, ['name']))

  if getv(from_object, ['partial_args']) is not None:
    raise ValueError('partial_args parameter is not supported in Gemini API.')

  if getv(from_object, ['will_continue']) is not None:
    raise ValueError('will_continue parameter is not supported in Gemini API.')

  return to_object


def _FunctionDeclaration_to_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['description']) is not None:
    setv(to_object, ['description'], getv(from_object, ['description']))

  if getv(from_object, ['name']) is not None:
    setv(to_object, ['name'], getv(from_object, ['name']))

  if getv(from_object, ['parameters']) is not None:
    setv(to_object, ['parameters'], getv(from_object, ['parameters']))

  if getv(from_object, ['parameters_json_schema']) is not None:
    setv(
        to_object,
        ['parametersJsonSchema'],
        getv(from_object, ['parameters_json_schema']),
    )

  if getv(from_object, ['response']) is not None:
    setv(to_object, ['response'], getv(from_object, ['response']))

  if getv(from_object, ['response_json_schema']) is not None:
    setv(
        to_object,
        ['responseJsonSchema'],
        getv(from_object, ['response_json_schema']),
    )

  if getv(from_object, ['behavior']) is not None:
    raise ValueError('behavior parameter is not supported in Vertex AI.')

  return to_object


def _GenerationConfig_to_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['model_selection_config']) is not None:
    setv(
        to_object,
        ['modelConfig'],
        getv(from_object, ['model_selection_config']),
    )

  if getv(from_object, ['response_json_schema']) is not None:
    setv(
        to_object,
        ['responseJsonSchema'],
        getv(from_object, ['response_json_schema']),
    )

  if getv(from_object, ['audio_timestamp']) is not None:
    setv(to_object, ['audioTimestamp'], getv(from_object, ['audio_timestamp']))

  if getv(from_object, ['candidate_count']) is not None:
    setv(to_object, ['candidateCount'], getv(from_object, ['candidate_count']))

  if getv(from_object, ['enable_affective_dialog']) is not None:
    setv(
        to_object,
        ['enableAffectiveDialog'],
        getv(from_object, ['enable_affective_dialog']),
    )

  if getv(from_object, ['frequency_penalty']) is not None:
    setv(
        to_object,
        ['frequencyPenalty'],
        getv(from_object, ['frequency_penalty']),
    )

  if getv(from_object, ['logprobs']) is not None:
    setv(to_object, ['logprobs'], getv(from_object, ['logprobs']))

  if getv(from_object, ['max_output_tokens']) is not None:
    setv(
        to_object, ['maxOutputTokens'], getv(from_object, ['max_output_tokens'])
    )

  if getv(from_object, ['media_resolution']) is not None:
    setv(
        to_object, ['mediaResolution'], getv(from_object, ['media_resolution'])
    )

  if getv(from_object, ['presence_penalty']) is not None:
    setv(
        to_object, ['presencePenalty'], getv(from_object, ['presence_penalty'])
    )

  if getv(from_object, ['response_logprobs']) is not None:
    setv(
        to_object,
        ['responseLogprobs'],
        getv(from_object, ['response_logprobs']),
    )

  if getv(from_object, ['response_mime_type']) is not None:
    setv(
        to_object,
        ['responseMimeType'],
        getv(from_object, ['response_mime_type']),
    )

  if getv(from_object, ['response_modalities']) is not None:
    setv(
        to_object,
        ['responseModalities'],
        getv(from_object, ['response_modalities']),
    )

  if getv(from_object, ['response_schema']) is not None:
    setv(to_object, ['responseSchema'], getv(from_object, ['response_schema']))

  if getv(from_object, ['routing_config']) is not None:
    setv(to_object, ['routingConfig'], getv(from_object, ['routing_config']))

  if getv(from_object, ['seed']) is not None:
    setv(to_object, ['seed'], getv(from_object, ['seed']))

  if getv(from_object, ['speech_config']) is not None:
    setv(to_object, ['speechConfig'], getv(from_object, ['speech_config']))

  if getv(from_object, ['stop_sequences']) is not None:
    setv(to_object, ['stopSequences'], getv(from_object, ['stop_sequences']))

  if getv(from_object, ['temperature']) is not None:
    setv(to_object, ['temperature'], getv(from_object, ['temperature']))

  if getv(from_object, ['thinking_config']) is not None:
    setv(to_object, ['thinkingConfig'], getv(from_object, ['thinking_config']))

  if getv(from_object, ['top_k']) is not None:
    setv(to_object, ['topK'], getv(from_object, ['top_k']))

  if getv(from_object, ['top_p']) is not None:
    setv(to_object, ['topP'], getv(from_object, ['top_p']))

  if getv(from_object, ['enable_enhanced_civic_answers']) is not None:
    raise ValueError(
        'enable_enhanced_civic_answers parameter is not supported in Vertex AI.'
    )

  return to_object


def _GoogleMaps_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['auth_config']) is not None:
    raise ValueError('auth_config parameter is not supported in Gemini API.')

  if getv(from_object, ['enable_widget']) is not None:
    setv(to_object, ['enableWidget'], getv(from_object, ['enable_widget']))

  return to_object


def _GoogleSearch_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['search_types']) is not None:
    setv(to_object, ['searchTypes'], getv(from_object, ['search_types']))

  if getv(from_object, ['exclude_domains']) is not None:
    raise ValueError(
        'exclude_domains parameter is not supported in Gemini API.'
    )

  if getv(from_object, ['blocking_confidence']) is not None:
    raise ValueError(
        'blocking_confidence parameter is not supported in Gemini API.'
    )

  if getv(from_object, ['time_range_filter']) is not None:
    setv(
        to_object, ['timeRangeFilter'], getv(from_object, ['time_range_filter'])
    )

  return to_object


def _LiveClientContent_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['turns']) is not None:
    setv(
        to_object,
        ['turns'],
        [
            _Content_to_mldev(item, to_object)
            for item in getv(from_object, ['turns'])
        ],
    )

  if getv(from_object, ['turn_complete']) is not None:
    setv(to_object, ['turnComplete'], getv(from_object, ['turn_complete']))

  return to_object


def _LiveClientMessage_to_mldev(
    api_client: BaseApiClient,
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['setup']) is not None:
    setv(
        to_object,
        ['setup'],
        _LiveClientSetup_to_mldev(
            api_client, getv(from_object, ['setup']), to_object
        ),
    )

  if getv(from_object, ['client_content']) is not None:
    setv(
        to_object,
        ['clientContent'],
        _LiveClientContent_to_mldev(
            getv(from_object, ['client_content']), to_object
        ),
    )

  if getv(from_object, ['realtime_input']) is not None:
    setv(
        to_object,
        ['realtimeInput'],
        _LiveClientRealtimeInput_to_mldev(
            getv(from_object, ['realtime_input']), to_object
        ),
    )

  if getv(from_object, ['tool_response']) is not None:
    setv(to_object, ['toolResponse'], getv(from_object, ['tool_response']))

  return to_object


def _LiveClientMessage_to_vertex(
    api_client: BaseApiClient,
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['setup']) is not None:
    setv(
        to_object,
        ['setup'],
        _LiveClientSetup_to_vertex(
            api_client, getv(from_object, ['setup']), to_object
        ),
    )

  if getv(from_object, ['client_content']) is not None:
    setv(to_object, ['clientContent'], getv(from_object, ['client_content']))

  if getv(from_object, ['realtime_input']) is not None:
    setv(
        to_object,
        ['realtimeInput'],
        _LiveClientRealtimeInput_to_vertex(
            getv(from_object, ['realtime_input']), to_object
        ),
    )

  if getv(from_object, ['tool_response']) is not None:
    setv(to_object, ['toolResponse'], getv(from_object, ['tool_response']))

  return to_object


def _LiveClientRealtimeInput_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['media_chunks']) is not None:
    setv(
        to_object,
        ['mediaChunks'],
        [
            _Blob_to_mldev(item, to_object)
            for item in getv(from_object, ['media_chunks'])
        ],
    )

  if getv(from_object, ['audio']) is not None:
    setv(
        to_object,
        ['audio'],
        _Blob_to_mldev(getv(from_object, ['audio']), to_object),
    )

  if getv(from_object, ['audio_stream_end']) is not None:
    setv(to_object, ['audioStreamEnd'], getv(from_object, ['audio_stream_end']))

  if getv(from_object, ['video']) is not None:
    setv(
        to_object,
        ['video'],
        _Blob_to_mldev(getv(from_object, ['video']), to_object),
    )

  if getv(from_object, ['text']) is not None:
    setv(to_object, ['text'], getv(from_object, ['text']))

  if getv(from_object, ['activity_start']) is not None:
    setv(to_object, ['activityStart'], getv(from_object, ['activity_start']))

  if getv(from_object, ['activity_end']) is not None:
    setv(to_object, ['activityEnd'], getv(from_object, ['activity_end']))

  return to_object


def _LiveClientRealtimeInput_to_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['media_chunks']) is not None:
    setv(
        to_object,
        ['mediaChunks'],
        [item for item in getv(from_object, ['media_chunks'])],
    )

  if getv(from_object, ['audio']) is not None:
    setv(to_object, ['audio'], getv(from_object, ['audio']))

  if getv(from_object, ['audio_stream_end']) is not None:
    raise ValueError(
        'audio_stream_end parameter is not supported in Vertex AI.'
    )

  if getv(from_object, ['video']) is not None:
    setv(to_object, ['video'], getv(from_object, ['video']))

  if getv(from_object, ['text']) is not None:
    setv(to_object, ['text'], getv(from_object, ['text']))

  if getv(from_object, ['activity_start']) is not None:
    setv(to_object, ['activityStart'], getv(from_object, ['activity_start']))

  if getv(from_object, ['activity_end']) is not None:
    setv(to_object, ['activityEnd'], getv(from_object, ['activity_end']))

  return to_object


def _LiveClientSetup_to_mldev(
    api_client: BaseApiClient,
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['model']) is not None:
    setv(to_object, ['model'], getv(from_object, ['model']))

  if getv(from_object, ['generation_config']) is not None:
    setv(
        to_object,
        ['generationConfig'],
        getv(from_object, ['generation_config']),
    )

  if getv(from_object, ['system_instruction']) is not None:
    setv(
        to_object,
        ['systemInstruction'],
        _Content_to_mldev(
            t.t_content(getv(from_object, ['system_instruction'])), to_object
        ),
    )

  if getv(from_object, ['tools']) is not None:
    setv(
        to_object,
        ['tools'],
        [
            _Tool_to_mldev(t.t_tool(api_client, item), to_object)
            for item in t.t_tools(api_client, getv(from_object, ['tools']))
        ],
    )

  if getv(from_object, ['session_resumption']) is not None:
    setv(
        to_object,
        ['sessionResumption'],
        _SessionResumptionConfig_to_mldev(
            getv(from_object, ['session_resumption']), to_object
        ),
    )

  if getv(from_object, ['context_window_compression']) is not None:
    setv(
        to_object,
        ['contextWindowCompression'],
        getv(from_object, ['context_window_compression']),
    )

  if getv(from_object, ['input_audio_transcription']) is not None:
    setv(
        to_object,
        ['inputAudioTranscription'],
        getv(from_object, ['input_audio_transcription']),
    )

  if getv(from_object, ['output_audio_transcription']) is not None:
    setv(
        to_object,
        ['outputAudioTranscription'],
        getv(from_object, ['output_audio_transcription']),
    )

  if getv(from_object, ['proactivity']) is not None:
    setv(to_object, ['proactivity'], getv(from_object, ['proactivity']))

  if getv(from_object, ['explicit_vad_signal']) is not None:
    raise ValueError(
        'explicit_vad_signal parameter is not supported in Gemini API.'
    )

  return to_object


def _LiveClientSetup_to_vertex(
    api_client: BaseApiClient,
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['model']) is not None:
    setv(to_object, ['model'], getv(from_object, ['model']))

  if getv(from_object, ['generation_config']) is not None:
    setv(
        to_object,
        ['generationConfig'],
        _GenerationConfig_to_vertex(
            getv(from_object, ['generation_config']), to_object
        ),
    )

  if getv(from_object, ['system_instruction']) is not None:
    setv(
        to_object,
        ['systemInstruction'],
        t.t_content(getv(from_object, ['system_instruction'])),
    )

  if getv(from_object, ['tools']) is not None:
    setv(
        to_object,
        ['tools'],
        [
            _Tool_to_vertex(t.t_tool(api_client, item), to_object)
            for item in t.t_tools(api_client, getv(from_object, ['tools']))
        ],
    )

  if getv(from_object, ['session_resumption']) is not None:
    setv(
        to_object,
        ['sessionResumption'],
        getv(from_object, ['session_resumption']),
    )

  if getv(from_object, ['context_window_compression']) is not None:
    setv(
        to_object,
        ['contextWindowCompression'],
        getv(from_object, ['context_window_compression']),
    )

  if getv(from_object, ['input_audio_transcription']) is not None:
    setv(
        to_object,
        ['inputAudioTranscription'],
        getv(from_object, ['input_audio_transcription']),
    )

  if getv(from_object, ['output_audio_transcription']) is not None:
    setv(
        to_object,
        ['outputAudioTranscription'],
        getv(from_object, ['output_audio_transcription']),
    )

  if getv(from_object, ['proactivity']) is not None:
    setv(to_object, ['proactivity'], getv(from_object, ['proactivity']))

  if getv(from_object, ['explicit_vad_signal']) is not None:
    setv(
        to_object,
        ['explicitVadSignal'],
        getv(from_object, ['explicit_vad_signal']),
    )

  return to_object


def _LiveConnectConfig_to_mldev(
    api_client: BaseApiClient,
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['generation_config']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig'],
        getv(from_object, ['generation_config']),
    )

  if getv(from_object, ['response_modalities']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'responseModalities'],
        getv(from_object, ['response_modalities']),
    )

  if getv(from_object, ['temperature']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'temperature'],
        getv(from_object, ['temperature']),
    )

  if getv(from_object, ['top_p']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'topP'],
        getv(from_object, ['top_p']),
    )

  if getv(from_object, ['top_k']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'topK'],
        getv(from_object, ['top_k']),
    )

  if getv(from_object, ['max_output_tokens']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'maxOutputTokens'],
        getv(from_object, ['max_output_tokens']),
    )

  if getv(from_object, ['media_resolution']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'mediaResolution'],
        getv(from_object, ['media_resolution']),
    )

  if getv(from_object, ['seed']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'seed'],
        getv(from_object, ['seed']),
    )

  if getv(from_object, ['speech_config']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'speechConfig'],
        t.t_live_speech_config(getv(from_object, ['speech_config'])),
    )

  if getv(from_object, ['thinking_config']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'thinkingConfig'],
        getv(from_object, ['thinking_config']),
    )

  if getv(from_object, ['enable_affective_dialog']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'enableAffectiveDialog'],
        getv(from_object, ['enable_affective_dialog']),
    )

  if getv(from_object, ['system_instruction']) is not None:
    setv(
        parent_object,
        ['setup', 'systemInstruction'],
        _Content_to_mldev(
            t.t_content(getv(from_object, ['system_instruction'])), to_object
        ),
    )

  if getv(from_object, ['tools']) is not None:
    setv(
        parent_object,
        ['setup', 'tools'],
        [
            _Tool_to_mldev(t.t_tool(api_client, item), to_object)
            for item in t.t_tools(api_client, getv(from_object, ['tools']))
        ],
    )

  if getv(from_object, ['session_resumption']) is not None:
    setv(
        parent_object,
        ['setup', 'sessionResumption'],
        _SessionResumptionConfig_to_mldev(
            getv(from_object, ['session_resumption']), to_object
        ),
    )

  if getv(from_object, ['input_audio_transcription']) is not None:
    setv(
        parent_object,
        ['setup', 'inputAudioTranscription'],
        getv(from_object, ['input_audio_transcription']),
    )

  if getv(from_object, ['output_audio_transcription']) is not None:
    setv(
        parent_object,
        ['setup', 'outputAudioTranscription'],
        getv(from_object, ['output_audio_transcription']),
    )

  if getv(from_object, ['realtime_input_config']) is not None:
    setv(
        parent_object,
        ['setup', 'realtimeInputConfig'],
        getv(from_object, ['realtime_input_config']),
    )

  if getv(from_object, ['context_window_compression']) is not None:
    setv(
        parent_object,
        ['setup', 'contextWindowCompression'],
        getv(from_object, ['context_window_compression']),
    )

  if getv(from_object, ['proactivity']) is not None:
    setv(
        parent_object,
        ['setup', 'proactivity'],
        getv(from_object, ['proactivity']),
    )

  if getv(from_object, ['explicit_vad_signal']) is not None:
    raise ValueError(
        'explicit_vad_signal parameter is not supported in Gemini API.'
    )

  return to_object


def _LiveConnectConfig_to_vertex(
    api_client: BaseApiClient,
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}

  if getv(from_object, ['generation_config']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig'],
        _GenerationConfig_to_vertex(
            getv(from_object, ['generation_config']), to_object
        ),
    )

  if getv(from_object, ['response_modalities']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'responseModalities'],
        getv(from_object, ['response_modalities']),
    )

  if getv(from_object, ['temperature']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'temperature'],
        getv(from_object, ['temperature']),
    )

  if getv(from_object, ['top_p']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'topP'],
        getv(from_object, ['top_p']),
    )

  if getv(from_object, ['top_k']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'topK'],
        getv(from_object, ['top_k']),
    )

  if getv(from_object, ['max_output_tokens']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'maxOutputTokens'],
        getv(from_object, ['max_output_tokens']),
    )

  if getv(from_object, ['media_resolution']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'mediaResolution'],
        getv(from_object, ['media_resolution']),
    )

  if getv(from_object, ['seed']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'seed'],
        getv(from_object, ['seed']),
    )

  if getv(from_object, ['speech_config']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'speechConfig'],
        t.t_live_speech_config(getv(from_object, ['speech_config'])),
    )

  if getv(from_object, ['thinking_config']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'thinkingConfig'],
        getv(from_object, ['thinking_config']),
    )

  if getv(from_object, ['enable_affective_dialog']) is not None:
    setv(
        parent_object,
        ['setup', 'generationConfig', 'enableAffectiveDialog'],
        getv(from_object, ['enable_affective_dialog']),
    )

  if getv(from_object, ['system_instruction']) is not None:
    setv(
        parent_object,
        ['setup', 'systemInstruction'],
        t.t_content(getv(from_object, ['system_instruction'])),
    )

  if getv(from_object, ['tools']) is not None:
    setv(
        parent_object,
        ['setup', 'tools'],
        [
            _Tool_to_vertex(t.t_tool(api_client, item), to_object)
            for item in t.t_tools(api_client, getv(from_object, ['tools']))
        ],
    )

  if getv(from_object, ['session_resumption']) is not None:
    setv(
        parent_object,
        ['setup', 'sessionResumption'],
        getv(from_object, ['session_resumption']),
    )

  if getv(from_object, ['input_audio_transcription']) is not None:
    setv(
        parent_object,
        ['setup', 'inputAudioTranscription'],
        getv(from_object, ['input_audio_transcription']),
    )

  if getv(from_object, ['output_audio_transcription']) is not None:
    setv(
        parent_object,
        ['setup', 'outputAudioTranscription'],
        getv(from_object, ['output_audio_transcription']),
    )

  if getv(from_object, ['realtime_input_config']) is not None:
    setv(
        parent_object,
        ['setup', 'realtimeInputConfig'],
        getv(from_object, ['realtime_input_config']),
    )

  if getv(from_object, ['context_window_compression']) is not None:
    setv(
        parent_object,
        ['setup', 'contextWindowCompression'],
        getv(from_object, ['context_window_compression']),
    )

  if getv(from_object, ['proactivity']) is not None:
    setv(
        parent_object,
        ['setup', 'proactivity'],
        getv(from_object, ['proactivity']),
    )

  if getv(from_object, ['explicit_vad_signal']) is not None:
    setv(
        parent_object,
        ['setup', 'explicitVadSignal'],
        getv(from_object, ['explicit_vad_signal']),
    )

  return to_object


def _LiveConnectParameters_to_mldev(
    api_client: BaseApiClient,
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['model']) is not None:
    setv(
        to_object,
        ['setup', 'model'],
        t.t_model(api_client, getv(from_object, ['model'])),
    )

  if getv(from_object, ['config']) is not None:
    setv(
        to_object,
        ['config'],
        _LiveConnectConfig_to_mldev(
            api_client, getv(from_object, ['config']), to_object
        ),
    )

  return to_object


def _LiveConnectParameters_to_vertex(
    api_client: BaseApiClient,
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['model']) is not None:
    setv(
        to_object,
        ['setup', 'model'],
        t.t_model(api_client, getv(from_object, ['model'])),
    )

  if getv(from_object, ['config']) is not None:
    setv(
        to_object,
        ['config'],
        _LiveConnectConfig_to_vertex(
            api_client, getv(from_object, ['config']), to_object
        ),
    )

  return to_object


def _LiveMusicClientMessage_to_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['setup']) is not None:
    raise ValueError('setup parameter is not supported in Vertex AI.')

  if getv(from_object, ['client_content']) is not None:
    raise ValueError('client_content parameter is not supported in Vertex AI.')

  if getv(from_object, ['music_generation_config']) is not None:
    raise ValueError(
        'music_generation_config parameter is not supported in Vertex AI.'
    )

  if getv(from_object, ['playback_control']) is not None:
    raise ValueError(
        'playback_control parameter is not supported in Vertex AI.'
    )

  return to_object


def _LiveMusicConnectParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['model']) is not None:
    setv(to_object, ['setup', 'model'], getv(from_object, ['model']))

  return to_object


def _LiveMusicConnectParameters_to_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['model']) is not None:
    raise ValueError('model parameter is not supported in Vertex AI.')

  return to_object


def _LiveMusicSetConfigParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['music_generation_config']) is not None:
    setv(
        to_object,
        ['musicGenerationConfig'],
        getv(from_object, ['music_generation_config']),
    )

  return to_object


def _LiveMusicSetConfigParameters_to_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['music_generation_config']) is not None:
    raise ValueError(
        'music_generation_config parameter is not supported in Vertex AI.'
    )

  return to_object


def _LiveMusicSetWeightedPromptsParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['weighted_prompts']) is not None:
    setv(
        to_object,
        ['weightedPrompts'],
        [item for item in getv(from_object, ['weighted_prompts'])],
    )

  return to_object


def _LiveMusicSetWeightedPromptsParameters_to_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['weighted_prompts']) is not None:
    raise ValueError(
        'weighted_prompts parameter is not supported in Vertex AI.'
    )

  return to_object


def _LiveSendRealtimeInputParameters_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['media']) is not None:
    setv(
        to_object,
        ['mediaChunks'],
        [
            _Blob_to_mldev(item, to_object)
            for item in t.t_blobs(getv(from_object, ['media']))
        ],
    )

  if getv(from_object, ['audio']) is not None:
    setv(
        to_object,
        ['audio'],
        _Blob_to_mldev(t.t_audio_blob(getv(from_object, ['audio'])), to_object),
    )

  if getv(from_object, ['audio_stream_end']) is not None:
    setv(to_object, ['audioStreamEnd'], getv(from_object, ['audio_stream_end']))

  if getv(from_object, ['video']) is not None:
    setv(
        to_object,
        ['video'],
        _Blob_to_mldev(t.t_image_blob(getv(from_object, ['video'])), to_object),
    )

  if getv(from_object, ['text']) is not None:
    setv(to_object, ['text'], getv(from_object, ['text']))

  if getv(from_object, ['activity_start']) is not None:
    setv(to_object, ['activityStart'], getv(from_object, ['activity_start']))

  if getv(from_object, ['activity_end']) is not None:
    setv(to_object, ['activityEnd'], getv(from_object, ['activity_end']))

  return to_object


def _LiveSendRealtimeInputParameters_to_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['media']) is not None:
    setv(
        to_object,
        ['mediaChunks'],
        [item for item in t.t_blobs(getv(from_object, ['media']))],
    )

  if getv(from_object, ['audio']) is not None:
    setv(to_object, ['audio'], t.t_audio_blob(getv(from_object, ['audio'])))

  if getv(from_object, ['audio_stream_end']) is not None:
    setv(to_object, ['audioStreamEnd'], getv(from_object, ['audio_stream_end']))

  if getv(from_object, ['video']) is not None:
    setv(to_object, ['video'], t.t_image_blob(getv(from_object, ['video'])))

  if getv(from_object, ['text']) is not None:
    setv(to_object, ['text'], getv(from_object, ['text']))

  if getv(from_object, ['activity_start']) is not None:
    setv(to_object, ['activityStart'], getv(from_object, ['activity_start']))

  if getv(from_object, ['activity_end']) is not None:
    setv(to_object, ['activityEnd'], getv(from_object, ['activity_end']))

  return to_object


def _LiveServerMessage_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['setupComplete']) is not None:
    setv(to_object, ['setup_complete'], getv(from_object, ['setupComplete']))

  if getv(from_object, ['serverContent']) is not None:
    setv(to_object, ['server_content'], getv(from_object, ['serverContent']))

  if getv(from_object, ['toolCall']) is not None:
    setv(to_object, ['tool_call'], getv(from_object, ['toolCall']))

  if getv(from_object, ['toolCallCancellation']) is not None:
    setv(
        to_object,
        ['tool_call_cancellation'],
        getv(from_object, ['toolCallCancellation']),
    )

  if getv(from_object, ['usageMetadata']) is not None:
    setv(to_object, ['usage_metadata'], getv(from_object, ['usageMetadata']))

  if getv(from_object, ['goAway']) is not None:
    setv(to_object, ['go_away'], getv(from_object, ['goAway']))

  if getv(from_object, ['sessionResumptionUpdate']) is not None:
    setv(
        to_object,
        ['session_resumption_update'],
        getv(from_object, ['sessionResumptionUpdate']),
    )

  if getv(from_object, ['voiceActivityDetectionSignal']) is not None:
    setv(
        to_object,
        ['voice_activity_detection_signal'],
        getv(from_object, ['voiceActivityDetectionSignal']),
    )

  if getv(from_object, ['voiceActivity']) is not None:
    setv(
        to_object,
        ['voice_activity'],
        _VoiceActivity_from_mldev(
            getv(from_object, ['voiceActivity']), to_object
        ),
    )

  return to_object


def _LiveServerMessage_from_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['setupComplete']) is not None:
    setv(to_object, ['setup_complete'], getv(from_object, ['setupComplete']))

  if getv(from_object, ['serverContent']) is not None:
    setv(to_object, ['server_content'], getv(from_object, ['serverContent']))

  if getv(from_object, ['toolCall']) is not None:
    setv(to_object, ['tool_call'], getv(from_object, ['toolCall']))

  if getv(from_object, ['toolCallCancellation']) is not None:
    setv(
        to_object,
        ['tool_call_cancellation'],
        getv(from_object, ['toolCallCancellation']),
    )

  if getv(from_object, ['usageMetadata']) is not None:
    setv(
        to_object,
        ['usage_metadata'],
        _UsageMetadata_from_vertex(
            getv(from_object, ['usageMetadata']), to_object
        ),
    )

  if getv(from_object, ['goAway']) is not None:
    setv(to_object, ['go_away'], getv(from_object, ['goAway']))

  if getv(from_object, ['sessionResumptionUpdate']) is not None:
    setv(
        to_object,
        ['session_resumption_update'],
        getv(from_object, ['sessionResumptionUpdate']),
    )

  if getv(from_object, ['voiceActivityDetectionSignal']) is not None:
    setv(
        to_object,
        ['voice_activity_detection_signal'],
        getv(from_object, ['voiceActivityDetectionSignal']),
    )

  if getv(from_object, ['voiceActivity']) is not None:
    setv(
        to_object,
        ['voice_activity'],
        _VoiceActivity_from_vertex(
            getv(from_object, ['voiceActivity']), to_object
        ),
    )

  return to_object


def _Part_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['media_resolution']) is not None:
    setv(
        to_object, ['mediaResolution'], getv(from_object, ['media_resolution'])
    )

  if getv(from_object, ['code_execution_result']) is not None:
    setv(
        to_object,
        ['codeExecutionResult'],
        getv(from_object, ['code_execution_result']),
    )

  if getv(from_object, ['executable_code']) is not None:
    setv(to_object, ['executableCode'], getv(from_object, ['executable_code']))

  if getv(from_object, ['file_data']) is not None:
    setv(
        to_object,
        ['fileData'],
        _FileData_to_mldev(getv(from_object, ['file_data']), to_object),
    )

  if getv(from_object, ['function_call']) is not None:
    setv(
        to_object,
        ['functionCall'],
        _FunctionCall_to_mldev(getv(from_object, ['function_call']), to_object),
    )

  if getv(from_object, ['function_response']) is not None:
    setv(
        to_object,
        ['functionResponse'],
        getv(from_object, ['function_response']),
    )

  if getv(from_object, ['inline_data']) is not None:
    setv(
        to_object,
        ['inlineData'],
        _Blob_to_mldev(getv(from_object, ['inline_data']), to_object),
    )

  if getv(from_object, ['text']) is not None:
    setv(to_object, ['text'], getv(from_object, ['text']))

  if getv(from_object, ['thought']) is not None:
    setv(to_object, ['thought'], getv(from_object, ['thought']))

  if getv(from_object, ['thought_signature']) is not None:
    setv(
        to_object,
        ['thoughtSignature'],
        getv(from_object, ['thought_signature']),
    )

  if getv(from_object, ['video_metadata']) is not None:
    setv(to_object, ['videoMetadata'], getv(from_object, ['video_metadata']))

  return to_object


def _SessionResumptionConfig_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['handle']) is not None:
    setv(to_object, ['handle'], getv(from_object, ['handle']))

  if getv(from_object, ['transparent']) is not None:
    raise ValueError('transparent parameter is not supported in Gemini API.')

  return to_object


def _Tool_to_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['retrieval']) is not None:
    raise ValueError('retrieval parameter is not supported in Gemini API.')

  if getv(from_object, ['computer_use']) is not None:
    setv(to_object, ['computerUse'], getv(from_object, ['computer_use']))

  if getv(from_object, ['file_search']) is not None:
    setv(to_object, ['fileSearch'], getv(from_object, ['file_search']))

  if getv(from_object, ['google_search']) is not None:
    setv(
        to_object,
        ['googleSearch'],
        _GoogleSearch_to_mldev(getv(from_object, ['google_search']), to_object),
    )

  if getv(from_object, ['code_execution']) is not None:
    setv(to_object, ['codeExecution'], getv(from_object, ['code_execution']))

  if getv(from_object, ['enterprise_web_search']) is not None:
    raise ValueError(
        'enterprise_web_search parameter is not supported in Gemini API.'
    )

  if getv(from_object, ['function_declarations']) is not None:
    setv(
        to_object,
        ['functionDeclarations'],
        [item for item in getv(from_object, ['function_declarations'])],
    )

  if getv(from_object, ['google_maps']) is not None:
    setv(
        to_object,
        ['googleMaps'],
        _GoogleMaps_to_mldev(getv(from_object, ['google_maps']), to_object),
    )

  if getv(from_object, ['google_search_retrieval']) is not None:
    setv(
        to_object,
        ['googleSearchRetrieval'],
        getv(from_object, ['google_search_retrieval']),
    )

  if getv(from_object, ['url_context']) is not None:
    setv(to_object, ['urlContext'], getv(from_object, ['url_context']))

  if getv(from_object, ['mcp_servers']) is not None:
    setv(
        to_object,
        ['mcpServers'],
        [item for item in getv(from_object, ['mcp_servers'])],
    )

  return to_object


def _Tool_to_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['retrieval']) is not None:
    setv(to_object, ['retrieval'], getv(from_object, ['retrieval']))

  if getv(from_object, ['computer_use']) is not None:
    setv(to_object, ['computerUse'], getv(from_object, ['computer_use']))

  if getv(from_object, ['file_search']) is not None:
    raise ValueError('file_search parameter is not supported in Vertex AI.')

  if getv(from_object, ['google_search']) is not None:
    setv(to_object, ['googleSearch'], getv(from_object, ['google_search']))

  if getv(from_object, ['code_execution']) is not None:
    setv(to_object, ['codeExecution'], getv(from_object, ['code_execution']))

  if getv(from_object, ['enterprise_web_search']) is not None:
    setv(
        to_object,
        ['enterpriseWebSearch'],
        getv(from_object, ['enterprise_web_search']),
    )

  if getv(from_object, ['function_declarations']) is not None:
    setv(
        to_object,
        ['functionDeclarations'],
        [
            _FunctionDeclaration_to_vertex(item, to_object)
            for item in getv(from_object, ['function_declarations'])
        ],
    )

  if getv(from_object, ['google_maps']) is not None:
    setv(to_object, ['googleMaps'], getv(from_object, ['google_maps']))

  if getv(from_object, ['google_search_retrieval']) is not None:
    setv(
        to_object,
        ['googleSearchRetrieval'],
        getv(from_object, ['google_search_retrieval']),
    )

  if getv(from_object, ['url_context']) is not None:
    setv(to_object, ['urlContext'], getv(from_object, ['url_context']))

  if getv(from_object, ['mcp_servers']) is not None:
    raise ValueError('mcp_servers parameter is not supported in Vertex AI.')

  return to_object


def _UsageMetadata_from_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['promptTokenCount']) is not None:
    setv(
        to_object,
        ['prompt_token_count'],
        getv(from_object, ['promptTokenCount']),
    )

  if getv(from_object, ['cachedContentTokenCount']) is not None:
    setv(
        to_object,
        ['cached_content_token_count'],
        getv(from_object, ['cachedContentTokenCount']),
    )

  if getv(from_object, ['candidatesTokenCount']) is not None:
    setv(
        to_object,
        ['response_token_count'],
        getv(from_object, ['candidatesTokenCount']),
    )

  if getv(from_object, ['toolUsePromptTokenCount']) is not None:
    setv(
        to_object,
        ['tool_use_prompt_token_count'],
        getv(from_object, ['toolUsePromptTokenCount']),
    )

  if getv(from_object, ['thoughtsTokenCount']) is not None:
    setv(
        to_object,
        ['thoughts_token_count'],
        getv(from_object, ['thoughtsTokenCount']),
    )

  if getv(from_object, ['totalTokenCount']) is not None:
    setv(
        to_object, ['total_token_count'], getv(from_object, ['totalTokenCount'])
    )

  if getv(from_object, ['promptTokensDetails']) is not None:
    setv(
        to_object,
        ['prompt_tokens_details'],
        [item for item in getv(from_object, ['promptTokensDetails'])],
    )

  if getv(from_object, ['cacheTokensDetails']) is not None:
    setv(
        to_object,
        ['cache_tokens_details'],
        [item for item in getv(from_object, ['cacheTokensDetails'])],
    )

  if getv(from_object, ['candidatesTokensDetails']) is not None:
    setv(
        to_object,
        ['response_tokens_details'],
        [item for item in getv(from_object, ['candidatesTokensDetails'])],
    )

  if getv(from_object, ['toolUsePromptTokensDetails']) is not None:
    setv(
        to_object,
        ['tool_use_prompt_tokens_details'],
        [item for item in getv(from_object, ['toolUsePromptTokensDetails'])],
    )

  if getv(from_object, ['trafficType']) is not None:
    setv(to_object, ['traffic_type'], getv(from_object, ['trafficType']))

  return to_object


def _VoiceActivity_from_mldev(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['type']) is not None:
    setv(to_object, ['voice_activity_type'], getv(from_object, ['type']))

  return to_object


def _VoiceActivity_from_vertex(
    from_object: Union[dict[str, Any], object],
    parent_object: Optional[dict[str, Any]] = None,
) -> dict[str, Any]:
  to_object: dict[str, Any] = {}
  if getv(from_object, ['type']) is not None:
    setv(to_object, ['voice_activity_type'], getv(from_object, ['type']))

  return to_object



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""[Experimental] Live Music API client."""

import contextlib
import json
import logging
from typing import AsyncIterator
import websockets

from . import _api_module
from . import _common
from . import _live_converters as live_converters
from . import _transformers as t
from . import errors
from . import types
from ._api_client import BaseApiClient
from ._common import set_value_by_path as setv

ConnectionClosed = websockets.ConnectionClosed

try:
  from websockets.asyncio.client import ClientConnection
  from websockets.asyncio.client import connect
except ModuleNotFoundError:
  from websockets.client import ClientConnection  # type: ignore
  from websockets.client import connect  # type: ignore

logger = logging.getLogger('google_genai.live_music')


class AsyncMusicSession:
  """[Experimental] AsyncMusicSession."""

  def __init__(self, api_client: BaseApiClient, websocket: ClientConnection):
    self._api_client = api_client
    self._ws = websocket

  async def set_weighted_prompts(
      self, prompts: list[types.WeightedPrompt]
  ) -> None:
    if self._api_client.vertexai:
      raise NotImplementedError(
          'Live music generation is not supported in Vertex AI.'
      )
    else:
      client_content_dict = {
          'weightedPrompts': [
              _common.convert_to_dict(prompt, convert_keys=True)
              for prompt in prompts
          ]
      }

    await self._ws.send(json.dumps({'clientContent': client_content_dict}))

  async def set_music_generation_config(
      self, config: types.LiveMusicGenerationConfig
  ) -> None:
    if self._api_client.vertexai:
      raise NotImplementedError(
          'Live music generation is not supported in Vertex AI.'
      )
    else:
      config_dict = _common.convert_to_dict(config, convert_keys=True)
    await self._ws.send(json.dumps({'musicGenerationConfig': config_dict}))

  async def _send_control_signal(
      self, playback_control: types.LiveMusicPlaybackControl
  ) -> None:
    if self._api_client.vertexai:
      raise NotImplementedError(
          'Live music generation is not supported in Vertex AI.'
      )
    else:
      playback_control_dict = {'playbackControl': playback_control.value}
      await self._ws.send(json.dumps(playback_control_dict))

  async def play(self) -> None:
    """Sends playback signal to start the music stream."""
    return await self._send_control_signal(types.LiveMusicPlaybackControl.PLAY)

  async def pause(self) -> None:
    """Sends a playback signal to pause the music stream."""
    return await self._send_control_signal(types.LiveMusicPlaybackControl.PAUSE)

  async def stop(self) -> None:
    """Sends a playback signal to stop the music stream.

    Resets the music generation context while retaining the current config.
    """
    return await self._send_control_signal(types.LiveMusicPlaybackControl.STOP)

  async def reset_context(self) -> None:
    """Reset the context (prompts retained) without stopping the music generation."""
    return await self._send_control_signal(
        types.LiveMusicPlaybackControl.RESET_CONTEXT
    )

  async def receive(self) -> AsyncIterator[types.LiveMusicServerMessage]:
    """Receive model responses from the server.

    Yields:
      The audio chunks from the server.
    """
    # TODO(b/365983264) Handle intermittent issues for the user.
    while result := await self._receive():
      yield result

  async def _receive(self) -> types.LiveMusicServerMessage:
    parameter_model = types.LiveMusicServerMessage()
    try:
      raw_response = await self._ws.recv(decode=False)
    except TypeError:
      raw_response = await self._ws.recv()  # type: ignore[assignment]
    except ConnectionClosed as e:
      if e.rcvd:
        code = e.rcvd.code
        reason = e.rcvd.reason
      else:
        code = 1006
        reason = websockets.frames.CLOSE_CODE_EXPLANATIONS.get(code, 'Abnormal closure.')
      errors.APIError.raise_error(code, reason, None)
    if raw_response:
      try:
        response = json.loads(raw_response)
      except json.decoder.JSONDecodeError:
        raise ValueError(f'Failed to parse response: {raw_response!r}')
    else:
      response = {}

    if self._api_client.vertexai:
      raise NotImplementedError('Live music generation is not supported in Vertex AI.')
    else:
      response_dict = response
    if not response_dict and response:
      # Error handling.
      errors.APIError.raise_error(response.get('code'), response, None)
    return types.LiveMusicServerMessage._from_response(
        response=response_dict, kwargs=parameter_model.model_dump()
    )

  async def close(self) -> None:
    """Closes the bi-directional stream and terminates the session."""
    await self._ws.close()


class AsyncLiveMusic(_api_module.BaseModule):
  """[Experimental] Live music module.

  Live music can be accessed via `client.aio.live.music`.
  """

  @_common.experimental_warning(
      'Realtime music generation is experimental and may change in future versions.'
  )
  @contextlib.asynccontextmanager
  async def connect(self, *, model: str) -> AsyncIterator[AsyncMusicSession]:
    """[Experimental] Connect to the live music server."""
    base_url = self._api_client._websocket_base_url()
    if isinstance(base_url, bytes):
      base_url = base_url.decode('utf-8')
    transformed_model = t.t_model(self._api_client, model)

    if self._api_client.api_key:
      api_key = self._api_client.api_key
      version = self._api_client._http_options.api_version
      uri = f'{base_url}/ws/google.ai.generativelanguage.{version}.GenerativeService.BidiGenerateMusic?key={api_key}'
      headers = self._api_client._http_options.headers

      # Only mldev supported
      request_dict = _common.convert_to_dict(
          live_converters._LiveMusicConnectParameters_to_mldev(
              from_object=types.LiveMusicConnectParameters(
                model=transformed_model,
              ).model_dump(exclude_none=True)
          )
      )

      setv(request_dict, ['setup', 'model'], transformed_model)

      request = json.dumps(request_dict)
    else:
      raise NotImplementedError('Live music generation is not supported in Vertex AI.')

    try:
      async with connect(uri, additional_headers=headers) as ws:
        await ws.send(request)
        logger.info(await ws.recv(decode=False))

        yield AsyncMusicSession(api_client=self._api_client, websocket=ws)
    except TypeError:
      # Try with the older websockets API
      async with connect(uri, extra_headers=headers) as ws:
        await ws.send(request)
        logger.info(await ws.recv())

        yield AsyncMusicSession(api_client=self._api_client, websocket=ws)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import logging
import pydantic

from ... import types


def _get_not_none_fields(model: pydantic.BaseModel) -> list[str]:
  """Returns field names in a Pydantic model whose values are not None."""
  return [
      field for field, value in model.model_dump().items() if value is not None
  ]


def test_empty_json_schema_conversion():
  """Test conversion of empty JSONSchema to Schema."""
  json_schema = types.JSONSchema()
  gemini_api_schema = types.Schema.from_json_schema(json_schema=json_schema)
  vertex_ai_schema = types.Schema.from_json_schema(
      json_schema=json_schema, api_option='VERTEX_AI'
  )

  assert gemini_api_schema == types.Schema()
  assert vertex_ai_schema == types.Schema()


def test_not_null_type_conversion():
  """Test conversion of JSONSchema.type to Schema.type"""
  json_schema_types = [
      'string',
      'number',
      'integer',
      'boolean',
      'array',
      'object',
  ]
  schema_types = [
      'STRING',
      'NUMBER',
      'INTEGER',
      'BOOLEAN',
      'ARRAY',
      'OBJECT',
  ]
  for json_schema_type, expected_type in zip(json_schema_types, schema_types):
    json_schema1 = types.JSONSchema(type=types.JSONSchemaType(json_schema_type))
    json_schema2 = types.JSONSchema(type=json_schema_type)
    gemini_api_schema1 = types.Schema.from_json_schema(json_schema=json_schema1)
    vertex_ai_schema1 = types.Schema.from_json_schema(
        json_schema=json_schema1, api_option='VERTEX_AI'
    )
    gemini_api_schema2 = types.Schema.from_json_schema(json_schema=json_schema2)
    vertex_ai_schema2 = types.Schema.from_json_schema(
        json_schema=json_schema2, api_option='VERTEX_AI'
    )

    gemini_api_not_none_field_name1 = _get_not_none_fields(gemini_api_schema1)
    vertex_api_not_none_field_name1 = _get_not_none_fields(vertex_ai_schema1)
    gemini_api_not_none_field_name2 = _get_not_none_fields(gemini_api_schema2)
    vertex_ai_not_none_field_name2 = _get_not_none_fields(vertex_ai_schema2)

    assert gemini_api_schema1.type == expected_type
    assert vertex_ai_schema1.type == expected_type
    assert gemini_api_schema2.type == expected_type
    assert vertex_ai_schema2.type == expected_type
    assert gemini_api_not_none_field_name1 == ['type']
    assert vertex_api_not_none_field_name1 == ['type']
    assert gemini_api_not_none_field_name2 == ['type']
    assert vertex_ai_not_none_field_name2 == ['type']


def test_nullable_conversion():
  """Test conversion of JSONSchema.nullable to Schema.nullable"""
  json_schema1 = types.JSONSchema(
      type=[types.JSONSchemaType('string'), types.JSONSchemaType('null')],
  )
  json_schema2 = types.JSONSchema(
      type=['string', 'null'],
  )
  gemini_api_schema1 = types.Schema.from_json_schema(json_schema=json_schema1)
  vertex_ai_schema1 = types.Schema.from_json_schema(
      json_schema=json_schema1, api_option='VERTEX_AI'
  )
  gemini_api_schema2 = types.Schema.from_json_schema(json_schema=json_schema2)
  vertex_ai_schema2 = types.Schema.from_json_schema(
      json_schema=json_schema2, api_option='VERTEX_AI'
  )
  gemini_api_not_none_field_names1 = _get_not_none_fields(gemini_api_schema1)
  vertex_ai_not_none_field_names1 = _get_not_none_fields(vertex_ai_schema1)
  gemini_api_not_none_field_names2 = _get_not_none_fields(gemini_api_schema2)
  vertex_ai_not_none_field_names2 = _get_not_none_fields(vertex_ai_schema2)

  assert gemini_api_schema1.nullable
  assert vertex_ai_schema1.nullable
  assert gemini_api_schema2.nullable
  assert vertex_ai_schema2.nullable
  assert set(gemini_api_not_none_field_names1) == set(['type', 'nullable'])
  assert set(vertex_ai_not_none_field_names1) == set(['type', 'nullable'])
  assert set(gemini_api_not_none_field_names2) == set(['type', 'nullable'])
  assert set(vertex_ai_not_none_field_names2) == set(['type', 'nullable'])


def test_nullable_in_union_like_type_conversion():
  """Test conversion of JSONSchema.nullable to Schema.nullable"""
  json_schema1 = types.JSONSchema(
      type=[
          types.JSONSchemaType('string'),
          types.JSONSchemaType('null'),
          types.JSONSchemaType('object'),
          types.JSONSchemaType('number'),
          types.JSONSchemaType('array'),
          types.JSONSchemaType('boolean'),
          types.JSONSchemaType('integer'),
      ],
  )
  gemini_api_schema1 = types.Schema.from_json_schema(json_schema=json_schema1)
  vertex_ai_schema1 = types.Schema.from_json_schema(
      json_schema=json_schema1, api_option='VERTEX_AI'
  )
  gemini_api_not_none_field_names1 = _get_not_none_fields(gemini_api_schema1)
  vertex_ai_not_none_field_names1 = _get_not_none_fields(vertex_ai_schema1)
  json_schema2 = types.JSONSchema(
      type=[
          'string',
          'null',
          'object',
          'number',
          'array',
          'boolean',
          'integer',
      ]
  )
  gemini_api_schema2 = types.Schema.from_json_schema(json_schema=json_schema2)
  vertex_ai_schema2 = types.Schema.from_json_schema(
      json_schema=json_schema2, api_option='VERTEX_AI'
  )
  expected_schema = types.Schema(
      nullable=True,
      any_of=[
          types.Schema(type='STRING'),
          types.Schema(type='OBJECT'),
          types.Schema(type='NUMBER'),
          types.Schema(type='ARRAY'),
          types.Schema(type='BOOLEAN'),
          types.Schema(type='INTEGER'),
      ],
  )

  assert gemini_api_schema1 == expected_schema
  assert vertex_ai_schema1 == expected_schema
  assert gemini_api_schema2 == expected_schema
  assert vertex_ai_schema2 == expected_schema


def test_union_like_type_conversion_suite1():
  """Test conversion of JSONSchema.type to Schema.any_of"""
  json_schema = types.JSONSchema(
      type=[
          types.JSONSchemaType('string'),
          types.JSONSchemaType('object'),
          types.JSONSchemaType('null'),
      ],
      description='description',
      default='default',
      max_length=10,
      min_length=5,
      enum=['value1', 'value2'],
      format='format',
      pattern='pattern',
      title='title',
      min_properties=1,
      max_properties=2,
      required=['field1', 'field2'],
      properties={
          'field1': types.JSONSchema(type='string'),
          'field2': types.JSONSchema(type='integer'),
      },
  )
  actual_gemini_api_schema = types.Schema.from_json_schema(
      json_schema=json_schema
  )
  actual_vertex_ai_schema = types.Schema.from_json_schema(
      json_schema=json_schema, api_option='VERTEX_AI'
  )
  expected_schema = types.Schema(
      nullable=True,
      any_of=[
          types.Schema(
              type='STRING',
              description='description',
              max_length=10,
              min_length=5,
              enum=['value1', 'value2'],
              format='format',
              pattern='pattern',
              title='title',
          ),
          types.Schema(
              type='OBJECT',
              properties={
                  'field1': types.Schema(type='STRING'),
                  'field2': types.Schema(type='INTEGER'),
              },
              required=['field1', 'field2'],
              min_properties=1,
              max_properties=2,
              title='title',
              description='description',
          ),
      ],
  )

  assert actual_gemini_api_schema == expected_schema
  assert actual_vertex_ai_schema == expected_schema


def test_union_like_type_conversion_suite2():
  """Test conversion of JSONSchema.type to Schema.any_of"""
  json_schema = types.JSONSchema(
      type=[
          types.JSONSchemaType('integer'),
          types.JSONSchemaType('array'),
      ],
      description='description',
      items=types.JSONSchema(type='integer', maximum=2, minimum=1),
      min_items=1,
      max_items=2,
      title='title',
      enum=['1', '2'],
      maximum=2,
      minimum=1,
  )
  actual_gemini_api_schema = types.Schema.from_json_schema(
      json_schema=json_schema
  )
  actual_vertex_ai_schema = types.Schema.from_json_schema(
      json_schema=json_schema, api_option='VERTEX_AI'
  )
  expected_schema = types.Schema(
      any_of=[
          types.Schema(
              type='INTEGER',
              description='description',
              maximum=2,
              minimum=1,
              enum=['1', '2'],
              title='title',
          ),
          types.Schema(
              type='ARRAY',
              items=types.Schema(type='INTEGER', maximum=2, minimum=1),
              min_items=1,
              max_items=2,
              title='title',
              description='description',
          ),
      ],
  )

  assert actual_gemini_api_schema == expected_schema
  assert actual_vertex_ai_schema == expected_schema


def test_array_type_conversion():
  """Test conversion of JSONSchema.items to Schema.items"""
  json_schema = types.JSONSchema(
      type=types.JSONSchemaType('array'),
      items=types.JSONSchema(
          type='object',
          properties={
              'field1': types.JSONSchema(type='string'),
              'field2': types.JSONSchema(type='integer'),
          },
          required=['field1', 'field2'],
          min_properties=1,
          max_properties=2,
          title='title',
          description='description',
      ),
  )
  gemini_api_schema = types.Schema.from_json_schema(json_schema=json_schema)
  vertex_ai_schema = types.Schema.from_json_schema(
      json_schema=json_schema, api_option='VERTEX_AI'
  )
  expected_schema = types.Schema(
      type='ARRAY',
      items=types.Schema(
          type='OBJECT',
          properties={
              'field1': types.Schema(type='STRING'),
              'field2': types.Schema(type='INTEGER'),
          },
          required=['field1', 'field2'],
          min_properties=1,
          max_properties=2,
          title='title',
          description='description',
      ),
  )

  assert gemini_api_schema == expected_schema
  assert vertex_ai_schema == expected_schema


def test_complex_object_type_conversion():
  """Test conversion of JSONSchema.properties to Schema.properties"""
  json_schema = types.JSONSchema(
      type=types.JSONSchemaType('object'),
      properties={
          'field1': types.JSONSchema(
              type=['string', 'array', 'null'],
              description='description1',
              max_length=20,
              min_length=15,
              enum=['value1', 'value2'],
              format='format',
              pattern='pattern',
              title='title1',
              items=types.JSONSchema(type='integer', maximum=2, minimum=1),
              min_items=1,
              max_items=2,
          ),
          'field2': types.JSONSchema(type='integer'),
      },
      required=['field1', 'field2'],
      min_properties=1,
      max_properties=2,
      title='title',
      description='description',
  )
  gemini_api_schema = types.Schema.from_json_schema(json_schema=json_schema)
  vertex_ai_schema = types.Schema.from_json_schema(
      json_schema=json_schema, api_option='VERTEX_AI'
  )
  expected_schema = types.Schema(
      type='OBJECT',
      properties={
          'field1': types.Schema(
              nullable=True,
              any_of=[
                  types.Schema(
                      type='STRING',
                      description='description1',
                      max_length=20,
                      min_length=15,
                      enum=['value1', 'value2'],
                      format='format',
                      pattern='pattern',
                      title='title1',
                  ),
                  types.Schema(
                      type='ARRAY',
                      items=types.Schema(type='INTEGER', maximum=2, minimum=1),
                      min_items=1,
                      max_items=2,
                      title='title1',
                      description='description1',
                  ),
              ],
          ),
          'field2': types.Schema(type='INTEGER'),
      },
      required=['field1', 'field2'],
      min_properties=1,
      max_properties=2,
      title='title',
      description='description',
  )

  assert gemini_api_schema == expected_schema
  assert vertex_ai_schema == expected_schema


def test_from_json_schema_logs_only_once(caplog):
  """Test that the info message is logged only once across multiple from_json_schema calls."""
  from ... import types as types_module

  types_module._from_json_schema_warning_logged = False

  caplog.set_level(logging.INFO, logger='google_genai.types')

  json_schema1 = types_module.JSONSchema(type='string')
  schema1 = types_module.Schema.from_json_schema(json_schema=json_schema1)

  assert len(caplog.records) == 1
  assert 'Json Schema is now supported natively' in caplog.text
  assert 'response_json_schema' in caplog.text

  json_schema2 = types_module.JSONSchema(type='number')
  schema2 = types_module.Schema.from_json_schema(json_schema=json_schema2)

  assert len(caplog.records) == 1

  json_schema3 = types_module.JSONSchema(type='object')
  schema3 = types_module.Schema.from_json_schema(json_schema=json_schema3)

  assert len(caplog.records) == 1

  assert schema1.type == types_module.Type('STRING')
  assert schema2.type == types_module.Type('NUMBER')
  assert schema3.type == types.Type('OBJECT')

  types_module._from_json_schema_warning_logged = False
